let chatidr = null;
let ProductId = '9498b455-be7b-4faa-a0df-40ce51d6bed2';
let TenantId = '21ba4031-7db8-417b-8c67-e7a746aab588';
let BaseUri = 'https://taavia-back.taavia.com';
// Stage :
// let ProductId='b33ed475-fb34-456a-b137-e2f35f8dcd0c';
// let TenantId='df544f5b-199f-49b1-8347-50df5da103f8';
let settingsData = {};

// Convert 8-character hex to rgba format
function hexToRgba(hex) {
    if (hex.length !== 8) {
        console.error("Invalid hex color format. Expected 8 characters.");
        return null;
    }

    // Parse RGBA values from hex code
    const a = parseInt(hex.substring(0, 2), 16) / 255; // Alpha
    const r = parseInt(hex.substring(2, 4), 16);
    const g = parseInt(hex.substring(4, 6), 16);
    const b = parseInt(hex.substring(6, 8), 16);

    // Return rgba color format
    return `rgba(${r}, ${g}, ${b}, ${a})`;
}

// Async function to fetch settings data
async function getSetting() {
    const url = BaseUri + '/api/v1/settings/chat?productID=' + ProductId;

    try {
        const response = await fetch(url, {
            method: 'GET',
            headers: {
                'accept': 'text/plain',
                'tenantId': TenantId,
                'accept-language': 'fa-IR, en-US'
            }
        });

        if (!response.ok) {
            throw new Error(`HTTP error! status: ${response.status}`);
        }

        const data = await response.json();

        // Parse and format colors
        settingsData = {
            buttonSetting: {
                chatButtonPositionId: data?.buttonSetting?.chatButtonPositionId,
                color: hexToRgba(data?.buttonSetting?.color),
                icon: {
                    extension: data?.buttonSetting?.icon?.extension,
                    documentId: data?.buttonSetting?.icon?.documentId
                }
            },

            chatSetting: {
                userMessageColor: hexToRgba(data?.chatSetting?.userMessageColor),
                userMessageTextColor: hexToRgba(data?.chatSetting?.userMessageTextColor),
                responseMessageColor: hexToRgba(data?.chatSetting?.responseMessageColor),
                responseMessageTextColor: hexToRgba(data?.chatSetting?.responseMessageTextColor),
                fontId: data?.chatSetting?.fontId
            }
        };

        console.log("Formatted Settings:", settingsData);



        return settingsData;
    } catch (error) {
        console.error('Error fetching settings:', error);
        return null;
    }
}

getSetting();


async function ConvertToURL(documentId) {
    const apiEndpoint = `${BaseUri}/api/v1/storages/${documentId}`;
    const tenantId = "e90d8807-8a8b-4744-b8df-6ff285f6f3ba";

    try {
        const response = await fetch(apiEndpoint, {
            method: "GET",
            headers: {
                "tenantId": tenantId
            }
        });

        if (!response.ok) {
            throw new Error('Network response was not ok');
        }

        const blob = await response.blob(); // دریافت پاسخ به صورت blob
        const imageUrl = URL.createObjectURL(blob);
        return imageUrl; // برگرداندن URL تصویر
    } catch (error) {
        console.error("خطا در دریافت لینک تصویر:", error);
        return null;

    }
}



let connection = new signalR.HubConnectionBuilder()
    .withUrl(BaseUri + "/chat?tenantId=" + TenantId + "&productId=" + ProductId, {
        transport: signalR.HttpTransportType.WebSockets,
        withCredentials: false,
        skipNegotiation: true,
    })
    .build();


//Start Conection
async function RStart() {
    return connection.start()
        .then(function () {
            console.log("Connected to SignalR!");
            return connection.invoke("SetTenant", TenantId);
        })
        .then(function () {
            console.log("Tenant set successfully.");
        })
        .catch(function (err) {
            console.error("Error during process:", err.toString());
        });
}


// تابع برای ساخت اتصال کاربر
function RConnection(name, number) {
    return connection.invoke("CreateConnection", {
        "name": name,
        "phoneNumber": number
    }).then(result => {
        console.log('CreateConnection Result:', result);
        if (result && result.messageId) {
            console.log('Connection created successfully.');
        } else {
            console.log('Connection creation failed.');
        }
    }).catch(err => {
        console.error('Error in CreateConnection:', err.toString());
    });
}

// تابع برای ساخت چت جدید
function RCreateChat(PId) {
    return connection.invoke("CreateConversation", {
        "productId": PId
    }).then(result => {
        console.log('CreateConversation Result:', result);
        if (result && result.messageId) {
            console.log('Chat created successfully.');
        } else {
            console.log('Chat creation failed.');
        }
    }).catch(err => {
        console.error('Error in CreateConversation:', err.toString());
    });
}

function MessageIsDelivered(MID) {

    connection.invoke("MessageIsDelivered", MID
    ).then(result => {

        console.log('Seen : ', result);
        if (result && result.messageId) {
            console.log('Seened.');
        } else {
            console.log('Seen failed.');
        }
    }).catch(err => {
        console.error('Error in Seen:', err.toString());
    });
}

function MessageIsSeen(MID) {

    connection.invoke("MessageIsSeen", MID
    ).then(result => {

        console.log('Seen : ', result);
        if (result && result.messageId) {
            console.log('Seen.');
        } else {
            console.log('Seen failed.');
        }
    }).catch(err => {
        console.error('Error in Seen:', err.toString());
    });
}



// function RJoin() {
// let chat = localStorage.getItem('ChatID');
// console.log('RJoin');

// if (connection.state === "Connected") {
// return connection.invoke("JoinConversation", chat)
// .then(function() {
// console.log("JoinConversation successfully.");
// })
// .catch(function(err) {
// console.error("JoinConversation:", err.toString());
// });
// } else {
// connection.start().then(function() {
// console.log("Connection started.");
// return connection.invoke("JoinConversation", chat)
// .then(function() {
// console.log("JoinConversation successfully.");
// })
// .catch(function(err) {
// console.error("JoinConversation:", err.toString());
// });
// }).catch(function(err) {
// console.error("Error starting connection:", err.toString());
// });
// }
// }
//For Join



async function RJoin(CID) {
    try {
        console.log('RJoin');
        const result = await connection.invoke("JoinConversation", CID);

        if (result) {
            console.log('Join successfully:', result);
        } else {
            console.log('Join failed.');
        }
    } catch (err) {
        console.error('Error in Join:', err.toString());
    }
}


let userMessageTimeout;

// متد برای ارسال پیام
function RSendMessage(usermessage) {
    let ConversationId = localStorage.getItem('ChatID');
    let messageId = new Date().getTime(); // ایجاد شناسه منحصر به فرد برای پیام

    // وضعیت failureMessageSent را برای ارسال پیام تنظیم کن
    localStorage.setItem('failureMessageSent', 'true');

    // تایمر برای ارسال پیام پیش‌فرض
    userMessageTimeout = setTimeout(() => {
        let failureMessageSent = localStorage.getItem('failureMessageSent') === 'true';

        if (failureMessageSent) {
            console.log("No response from server. Sending fallback message.");
            // ارسال پیام پیش‌فرض در صورت عدم دریافت پاسخ از سرور
            ChatAgent("در حال حاضر تاویا در دسترس نمی‌باشد", new Date(), messageId, 2, 2, null);
            ShowTypingIndicator('off');
            DisableSendMsg(false);
        }
    }, 30000); // 2 ثانیه

    // ارسال پیام به سرور
    connection.invoke("SendMessage", {
        "text": usermessage,
        "conversationId": ConversationId
    }).then(result => {
        console.log('SendMessage Result:', result);
        if (result && result.messageId) {
            console.log('Message sent successfully.');
            // پیام با موفقیت ارسال شد، failureMessageSent را false قرار بده
            localStorage.setItem('failureMessageSent', 'false');
            clearTimeout(userMessageTimeout); // لغو تایمر در صورت دریافت پاسخ از سرور
        } else {
            console.log('Message send failed.');
            localStorage.setItem('failureMessageSent', 'true');
        }
    }).catch(err => {
        console.error('Error in SendMessage:', err.toString());
        // خطا در ارسال پیام، failureMessageSent را true قرار بده
        localStorage.setItem('failureMessageSent', 'true');
    });

}



// متد برای ارسال پیام صوتی
function RSendVoice(DI) {
    let ConversationId = localStorage.getItem('ChatID');
    let messageId = new Date().getTime(); // ایجاد شناسه منحصر به فرد برای پیام صوتی

    // قبل از ارسال پیام صوتی، failureMessageSent را true قرار بده
    localStorage.setItem('failureMessageSent', 'true');

    // منتظر بمان تا 2 ثانیه بعد از ارسال پیام
    let userMessageTimeout = setTimeout(() => {
        let failureMessageSent = localStorage.getItem('failureMessageSent') === 'true';

        if (failureMessageSent) {
            console.log("No response from server. Sending fallback message.");
            // ارسال پیام پیش‌فرض در صورت عدم پاسخ
            ChatAgent("در حال حاضر تاویا در دسترس نمی‌باشد", new Date(), messageId, 2, 2, null);
            ShowTypingIndicator('off');
            DisableSendMsg(false);
        }
    }, 30000); // 2 ثانیه

    // ارسال پیام صوتی به سرور
    connection.invoke("SendVoiceMessage", {
        "documentId": DI,
        "extension": 'audio/mp3',
        "conversationId": ConversationId
    }).then(result => {
        console.log('SendVoiceMessage Result:', result.documentId);
        // اگر پیام صوتی با موفقیت ارسال شد، failureMessageSent را false قرار بده
        localStorage.setItem('failureMessageSent', 'false');
        clearTimeout(userMessageTimeout); // در صورت پاسخ از سرور، تایمر را لغو کن
    }).catch(err => {
        console.error('Error in SendVoiceMessage:', err.toString());
        // اگر خطا در ارسال پیام صوتی پیش بیاید، failureMessageSent را true قرار بده
        localStorage.setItem('failureMessageSent', 'true');
    });
}


//For Like 
function RLike(MID) {

    connection.invoke("LikeMessage", MID
    ).then(result => {

        console.log('Like : ', result);
        if (result && result.messageId) {
            console.log('Like successfully.');
        } else {
            console.log('Like failed.');
        }
    }).catch(err => {
        console.error('Error in Like:', err.toString());
    });
}


//For Dislike
function RDisLike(MID) {

    connection.invoke("DislikeMessage", MID
    ).then(result => {

        console.log('Like : ', result);
        if (result && result.messageId) {
            console.log('disLike successfully.');
        } else {
            console.log('disLike failed.');
        }
    }).catch(err => {
        console.error('Error in disLike:', err.toString());
    });
}

//Remove Opinion
function RRemoveAction(MID) {

    connection.invoke("RemoveOpinion", MID
    ).then(result => {

        console.log('Like : ', result);
        if (result && result.messageId) {
            console.log('Removed');
        } else {
            console.log('notRemove');
        }
    }).catch(err => {
        console.error('Error in Remove:', err.toString());
    });
}


// دریافت پیام برای تنظیم conversationId
connection.on("SetConversationId", (message) => {
    if (message && message.length > 0) {
        chatidr = message;
        localStorage.setItem('ChatID', chatidr);
        console.log('Chat ID set:', chatidr);
    } else {
        console.log('SetConversationId received but no ID found.');
    }
});



connection.on("ReceiveMessage", (message) => {
    ShowTypingIndicator('off');
    DisableSendMsg(false);
    console.log("role: " + message.role);

    clearTimeout(userMessageTimeout); // لغو تایمر پیام پیش‌فرض

    localStorage.setItem('failureMessageSent', 'false');

    if (message.role === 1) {
        if (message.messageContent.media) {
            UserChat(message.messageContent.text, message.messageContent.dateTime, message.messageContent.id, message.messageContent.media.id);
        } else {
            UserChat(message.messageContent.text, message.messageContent.dateTime);
        }
    } else {
        if (message.messageContent.media) {
            ChatAgent(message.messageContent.text, message.messageContent.dateTime, message.messageContent.id, null, null, message.messageContent.media.id);
        } else {
            ChatAgent(message.messageContent.text, message.messageContent.dateTime, message.messageContent.id, null, null, null, null);
        }

        console.log("skdjsak : ", message.messageContent.id);
        MessageIsSeen(message.messageContent.id);
    }

    chatWrapper.scrollTop = chatWrapper.scrollHeight - 20;
});


// connection.on("ReceiveMessage", (message) => {
//     ShowTypingIndicator('off');
//     DisableSendMsg(false);
//     console.log("role: " + message.role);
//     if (message.role === 1) {
//         if (message.messageContent.media) {
//             UserChat(message.messageContent.text, message.messageContent.dateTime, message.messageContent.id, message.messageContent.media.documentId);
//         } else {
//             UserChat(message.messageContent.text, message.messageContent.dateTime);
//         }
//     } else {
//         if (message.messageContent.media) {
//             ChatAgent(message.messageContent.text, message.messageContent.dateTime, message.messageContent.id, null, null, message.messageContent.media.documentId);
//         } else {
//             ChatAgent(message.messageContent.text, message.messageContent.dateTime, message.messageContent.id, null, null, null, null);
//         }
//         console.log("skdjsak : ", message.messageContent.id);
//         MessageIsSeen(message.messageContent.id);
//     }


//     chatWrapper.scrollTop = chatWrapper.scrollHeight - 20;
// });


var metaTag = document.createElement('meta');
metaTag.name = 'viewport';
metaTag.content = 'width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no';

// Append it to the head of the document
document.head.appendChild(metaTag);



function addStyles() {
    var style = document.createElement('style');

    style.innerHTML = `
        @font-face {
            font-family: 'YekanBakh';
              src: url('https://cdn.jsdelivr.net/gh/haniyeroozmand/myfont/YekanBakhFaNum-Regular.woff') format('woff');
        }


        @keyframes slideInFromRight {
    0% {
        transform: translateX(100%);
        opacity: 0;
    }
    100% {
        transform: translateX(0);
        opacity: 1;
    }
}



/* Keyframe animation for fade-in */
@keyframes fadeIn {
    from {
        opacity: 0;
        transform: translateY(10px);
    }
    to {
        opacity: 1;
        transform: translateY(0);
    }
}

/* Keyframe animation for fade-out */
@keyframes fadeOut {
    from {
        opacity: 1;
        transform: translateY(0);
    }
    to {
        opacity: 0;
        transform: translateY(10px);
    }
}

        .chat-icon {
            background-color: #46C980;
            width: 35px;
            height: 35px;
            border-radius: 50%;
            padding: 10px;
            margin: 10px;
            cursor: pointer;
            position: fixed;
            right: 0;
            bottom: 0;
			display: flex;
			justify-content: center;
			z-index: 5000000;
            align-items:center;
            box-sizing: content-box;
            animation: slideInFromRight 0.5s ease-out;

        }
         
        img#ICON {
        width: 30px;
        height: 30px;
        } 

 
        .chatbox {
            width: 340px;
            height: 595px;
            background-color: white;
            // box-shadow: 2px 2px 3px rgba(198, 195, 195, 0.14);
            box-shadow: rgba(0, 0, 0, 0.05) 0px 0.48px 2.41px -0.38px, rgba(0, 0, 0, 0.17) 0px 4px 20px -0.75px;
            border-radius: 12px;
            padding: 10px 0px;
            position: fixed;
            right: 20px;
            bottom: 90px;
            display: none; 
            flex-direction: column;
            font-family: 'YekanBakh';
            border: 1px solid rgba(198, 195, 195, 0.14);
		   z-index: 5000000;
           direction: ltr;
             overflow: hidden;
             box-sizing: content-box;
                line-height: initial;
    transform:  translateY(10px);
    // transition: opacity 0.5s ease-in-out, transform 0.5s ease-in-out;
        }


.chatbox.active {
  display: flex;
    animation: fadeIn 0.5s ease-in-out forwards;
}

.chatbox.closing {
    animation: fadeOut 0.5s ease-in-out forwards;

}

        .chatbox-inside-section1 {
            box-shadow: 2px 2px 3px rgba(198, 195, 195, 0.14);
            display: flex;
            justify-content: space-between;
			height: 6vh;
        }

        .chatbox-inside-section2 {
            flex: 7;
            text-align: center;
            margin-top: 5px;
            display: flex;
            flex-direction: column;
            justify-content: start;
            overflow-y: hidden;
        }

        .chatbox-inside-section2 p {
            font-size: 12px;
            margin-top: 10px;
            font-weight: 600;
            text-align:center;
        }

        .chatbox-inside-section3 {
            min-height: 5.3vh;
            display: none;
            justify-content: space-between;
            align-items: center;
            padding: 0px 7px;
            margin: 5px 10px;
            border-radius: 12px;
            box-shadow: 2px 2px 10px rgba(198, 195, 195, 0.5);
            z-index:100000;
            background-color: white;
        }

        .font-style {
            font-size: 18px;
        }

        .inner-div-1 {
            display: grid;
            align-items: center;
            margin-left: 15px;
        }

        .inner-div-1:hover {
            cursor: pointer;
        }

        .inner-div-2 {
            display: grid;
            grid-template-columns: 50% 50%;
            gap: 15px;
            justify-content: center;
            align-content: center;
            align-items: center;
        }

        .inner-div-2 div img {
       width: 30px;
        } 

        .InnerInsideSection2Form {
            background-color: #EBEBEB;
            margin: 10px 30px;
            border-radius: 12px;
            display: flex;
            flex-direction: column;
            padding: 15px 20px 20px 20px;
            gap: 10px;
            align-items: center;
        }

        .InnerInsideSection2Form input {
            border: none;
            padding: 10px;
            border-radius: 12px;
            direction: rtl;
            width: 100%;
            background-color: white !important;
             font: initial;
             font-family: 'YekanBakh' !important;

        }

        .InnerInsideSection2Form button {
            width: 100px;
           padding: 4px 13px;
            border: none;
            font-family: 'YekanBakh' !important;
            border-radius: 12px;
            height: 30px;
           font-weight: 500;
            font: initial;
            margin: 0;
            font-size: 14px;
            cursor:pointer;


        }

        .InnerInsideSection2Form input:focus {
            outline: none;
            box-shadow: 2px 2px 3px rgba(198, 195, 195, 1);
        }

        .InnerInsideSection2Form input::placeholder {
    color: grey;
    font-size: 14px;
}

        .InnerInsideSection32 {
            width: 80%;
            display:flex;
            align-items: center;
        }

        .InnerInsideSection32 textarea {
            width: 100%;
            border: none;
            border-radius: 12px;
            direction: rtl;
            font-family: 'YekanBakh';
            font-size: 11px;
            align-content: center;
            resize:none !important;
            padding: 5px !important;
            box-shadow: none !important;

        }

        .InnerInsideSection32 textarea:focus {
            outline: none;
        }

        .InnerInsideSection31 {
            display: flex;
            gap: 8px;
        }

        .InnerInsideSection31 img {
            cursor: pointer;
        }

  .user-message{
    border-radius: 12px;
    margin: 5px;
    font-size: 11px;
    word-wrap: break-word;
    text-align: justify;
    max-width: 70%; 
    align-self: flex-end; 
    padding: 10px;
}

.bot-message {
    border-radius: 12px;
    margin: 5px;
    font-size: 11px;
    word-wrap: break-word;
    text-align: justify;
    max-width: 70%;
    align-self: flex-start; 
    direction: rtl;
    padding: 10px;
}

.like-dislike-icon{
    text-align: right; 
    display:flex;
    justify-content: flex-start;
    margin-left: 5px;

    
}

.chat-wrapper {
    display: flex ;
    overflow-y: scroll;
	flex-direction: column;
    background-color: #ffffff00;
}

     
   .message-container{
    flex: 7;
    text-align: center;
    display: flex;
    flex-direction: column;
    justify-content: start;
    padding: 5px;
}
   
.bot-message-container {
    flex: 7;
    display: flex;
    flex-direction: column;
    align-items: flex-start; /* Align children to the start (left) */
    padding: 5px;
}

    .message-timestamp, .bot-message-timestamp{
    font-size: 10px;
    text-align: right;
    overflow: hidden;
    direction: rtl;
    color: #949494;
    margin-right: 10px
    }

    .message-timestamp{
    margin-top: 2px;
  margin-right: 5px !important;
    }

.timestamp-statusicon{
    display: flex;
	flex-direction: row-reverse;
	justify-content: right;
	margin-right: 5px;
    align-items: center;
}

    .Bot-message-section{
    display:flex;
    flex-direction: column-reverse;
    }

.bot-reaction-timestamp-container {
    display: flex;
    flex-direction: row-reverse;
    align-self: flex-start; 
    gap: 5px;
    margin-left: 5px;
	align-items: center;
}

    .typing-indicator{
    // text-align: center;
	// direction: rtl;
	margin-bottom: 10px;
	// position: absolute;
	// bottom: 80px !important;
	// right: 35%;
     font-size: 13px;
    text-align: center;
	direction: rtl;
    color: #4a4a4a;
    }  


  .user-message audio {
   width: 230px !important;
    }

    audio{
    width: 230px !important;
    }

 audio::-webkit-media-controls-panel {
  background-color: #ebebeb;
  }


audio::-webkit-media-controls-toggle-closed-captions-button{
  display:none;
  }



.user-message:has(audio) {
  background-color: white;
}


.wavesurfer-container {
    height: 50px !important;
    order: 2;
}

.play-button svg , .pause-button svg{
    cursor: pointer;
    order: 1;
    width: 23px;
	fill: #ffffffb0;
}

.voice-btn-send {
    display: flex;
    width:80%;
    align-items: center;
    gap:10px;
    background-color: #0FDE71;
    border-radius: 12px;
    align-self: flex-end;
    padding: 0px 10px;
}


   span.voice-duration{
   order:3;
   font-size: 12px;
   margin-right: 5px;
   width: 30px;
   }


   .duration-display{
   font-size: 12px;
   }


.pause-button{
cursor: pointer;
display:none;
}


    .chatbox-overlay {
        position: absolute;
        top: 0;
        left: 0;
        width: 100%;
        height: 100%;
        background-color: rgba(255, 255, 255, 0.7);
        display: none;
        z-index: 100;
    }

    .disabled {
        pointer-events: none;
        opacity: 0.6;
    }

.chatbox-body {
    height: 510px;
    display: flex;
    flex-direction: column;
    justify-content: space-between;
}

.reaction-icon{
	cursor: pointer;
}

    .reaction-icon svg{
		 width: 15px;
	 }

    .reaction-container {
    display: flex;
    gap: 6px;
}
 
.Trash-icon{
	 width: 14px;
	 fill: rgb(80, 231, 151);
 }
 
  .recordvoice-icon {
	  width: 12px;
	   fill: rgb(80, 231, 151);
  }
 
 .send-icon {
	 width: 14px;
	  fill: rgb(80, 231, 151);
 }
 
.reaction-icon.active-like {
    fill: rgb(80, 231, 151); 
}

.reaction-icon.active-dislike {
    fill: #ff0000
}

.welcome-msg{
	  font-size: 11px;
    font-weight: 600;
    margin-top: 15px;
	text-align: center;
    direction : rtl;
    color: black;
	
}

.chat-wrapper::-webkit-scrollbar {
  width: 18px ;
}

.chat-wrapper::-webkit-scrollbar-track {
  background-color: transparent !important;
}


.chat-wrapper::-webkit-scrollbar-thumb {
  border-radius: 20px;
  border: 6px solid transparent;
  background-clip: content-box;
background-color: #e4e6e6 !important;
}

.chat-wrapper::-webkit-scrollbar-thumb:hover {

  background-color: #a8bbbf;

}
.sign-main-div {
    position: absolute;
    bottom: 0px;
    left: 50%;
    transform: translateX(-50%);
    width: 100% !important;
    display: flex;
    justify-content: center !important;
    border-top: 1px solid #eeeeee;
    background-color: #d3d3d342;
    align-items: center;
}

.sign {
    order: 12;
    font-size: 12px;
    text-align: center;
    color: #737373;
    border-radius: 12px;
    opacity: 1;
    transition: opacity 0.3s ease, transform 0.3s ease;
    bottom: 3px;
    padding: 8px;
    border-radius: 0px 0px 8px 8px !important;
}




.waveform-canvas{
	width: 70% ;
	height: 5.5vh;
	cursor: pointer;
	margin-top: 10px;
}
   .canvas-container {
	   height: 5vh;
   }

   .audio-container{
	padding: 5px;
    border-radius: 12px;
    display: flex;
    align-items: center;
    gap: 10px;
	margin-left: 70px;
	height: 4vh;
	justify-content: space-between;
   }



  


.audio-player {
    border-radius: 12px;
    padding: 5px;
    width: 100%;
    box-shadow: 0 3px 10px rgb(71 71 71 / 15%);
}


.controls {
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: 2px;
    margin: 3px;
}


.play-pause-btn {
    border: none;
    padding: 5px;
    border-radius: 10px;
    cursor: pointer;
   width: 30px;
   height: 25px;
   display: flex;
    justify-content: center;
    align-items: center;
    background-color: #ffffff00;
    margin: 0;
}




.progress-bar {
    flex-grow: 1;
    appearance: none;
    height: 6px;
    background: #ddd;
    border-radius: 5px;
    outline: none;
    margin: 0 5px;
    width: 100%;
}

.progress-bar::-webkit-slider-thumb {
    appearance: none;
    width: 12px;
    height: 12px;
    background-color: #fff;
    border-radius: 50%;
    cursor: pointer;
}

.progress-bar::-moz-range-thumb {
    width: 12px;
    height: 12px;
    background-color: #fff;
    border-radius: 50%;
    cursor: pointer;
}


.time {
    color: #fff;
    font-size: 10px;
}


audio {
    display: none;
}


button#playPauseBtn svg {
width: 12px;
}

.bot-message .audio-player .controls{
direction: ltr;
}
   @media (max-width:530px){
   .chatbox{
   width: 100%;
   height: 100%;
   right: 0;
   padding: 0px;
   top: 0;
   left: 0 !important;
   }

   canvas.frequency-visualizer{
   width: 100%;
   }
   
   .chatbox-body{
	   height: 88%;
   }
   
   }
.status-icon{
	width: 12px;
}

.status-icon svg path{
	fill: #949494;
}

.sign-text-class{
text-decoration: none;
}

.sign-logo{
width: 30px;
height: 16px;
}

.chat-wrapper * {
    box-sizing: revert;
}

.logo-class {
    order: 1;
}

.inner-div-2 p {
    margin: 0px !important;
}

.disabled {
  pointer-events: none; 
  opacity: 0.5;         
  cursor: not-allowed;   
}

.failure-msg{
    background-color: #f8d7da !important;
    border-color: #f5c6cb !important;
    color: #721c24 !important;
}

`;

    document.head.appendChild(style);
}


var chatBoxBody = document.createElement('div');
chatBoxBody.classList.add('chatbox-body');
var chatWrapper = document.createElement('div');
chatWrapper.classList.add('chat-wrapper');
const lang = document.documentElement.getAttribute('lang');
var companySignMainDiv = document.createElement('div');
var companySignMainAtag = document.createElement('a');
companySignMainAtag.href = 'https://taavia.com';
companySignMainDiv.classList.add('sign-main-div');
var companySign = document.createElement('div');
var SignLogo = document.createElement('img');
SignLogo.classList.add('sign-logo');
SignLogo.src = 'https://learn.taavsys.com/chat/icons/logo2.png';
companySign.innerHTML = 'قدرت گرفته از ';
companySignMainAtag.appendChild(companySignMainDiv);
companySignMainDiv.appendChild(companySign);
companySignMainDiv.appendChild(SignLogo);
companySign.classList.add('sign');
const botMessages = document.getElementsByClassName('bot-message-container');


let TypingIndicator = document.querySelector('.typing-indicator');
if (!TypingIndicator) {
    TypingIndicator = document.createElement('div');
    TypingIndicator.classList.add('typing-indicator');
    TypingIndicator.textContent = 'تاویا درحال پاسخگویی...';
    TypingIndicator.style.display = 'none'; // Initially hidden
    chatWrapper.appendChild(TypingIndicator);
}


  


function ShowTypingIndicator(flag) {
    if (flag === 'on') {
        TypingIndicator.style.display = 'block';
    } else {
        TypingIndicator.style.display = 'none';
    }
    chatWrapper.appendChild(TypingIndicator);
    chatWrapper.scrollTop = chatWrapper.scrollHeight;

}

function updateLastMessageMargin(isNearBottom, isVoiceMessage) {
    if (botMessages.length > 0) {
        const lastMessage = botMessages[botMessages.length - 1];
        const firstMessage = botMessages[0];

        // Check if the last message is also the first message
        if (lastMessage === firstMessage) {
            // Apply the margin for the first message
            lastMessage.style.setProperty('margin-bottom', isNearBottom ? '20px' : '0px');
        } else {
            // Apply the margin for the last message based on whether it's a voice message
            lastMessage.style.setProperty('margin-bottom', isVoiceMessage ? '50px' : (isNearBottom ? '10px' : '0px'));
        }
    }
}



function scrollToBottom() {
    chatWrapper.scrollTop = chatWrapper.scrollHeight;
}


chatWrapper.addEventListener('scroll', () => {
    const isNearBottom = chatWrapper.scrollTop + chatWrapper.clientHeight >= chatWrapper.scrollHeight - 20;

    // if (isNearBottom) {
    //     companySign.style.opacity = '1';
    // } else {
    //     companySign.style.opacity = '0';
    // }

    updateLastMessageMargin(isNearBottom, null);

});




function toggleChatBox() {
    const chatbox = document.querySelector('.chatbox');
    const chatIcon = document.querySelector('.chat-icon');

    if (chatbox) {
        if (chatbox.classList.contains('active')) {
            // Start fade-out effect
            chatbox.classList.remove('active');
            chatbox.classList.add('closing');

            setTimeout(() => {
                chatbox.style.display = 'none';
                chatbox.classList.remove('closing');
            }, 500);
        } else {
            // Make chatbox visible before starting the fade-in animation
            chatbox.style.display = 'flex';
            chatbox.classList.add('active');


        }

        // Smoothly hide/show the chat icon on small screens
        if (window.innerWidth <= 530) {
            chatIcon.style.opacity = chatbox.classList.contains('active') ? '0' : '1';
        }
    }
}



async function showaudio(docid, message, UserMessage) {
    if (docid) {
        try {
            const mediauri = await ConvertToURL(docid);
            if (mediauri) {
                // Custom audio player structure
                message.innerHTML = `
                    <div class="audio-player">
                        <audio id="audio" controls>
                            <source src="${mediauri}" type="audio/mp3">
                        </audio>
                        <div class="controls">
                            <button id="playPauseBtn" class="play-pause-btn">
                                <!-- Play icon (initial state) -->
                                <svg  xmlns="http://www.w3.org/2000/svg" viewBox="0 0 384 512">
                                    <path d="M64.7 52.7c-9.9-6-22.2-6.3-32.3-.6S16 68.4 16 80l0 352c0 11.6 6.3 22.2 16.3 27.9s22.5 5.4 32.3-.6l288-176c9.5-5.8 15.3-16.2 15.3-27.3s-5.8-21.5-15.3-27.3l-288-176zM24.5 38.1C39.7 29.6 58.2 30 73 39L361 215c14.3 8.7 23 24.2 23 41s-8.7 32.2-23 41L73 473c-14.8 9.1-33.4 9.4-48.5 .9S0 449.4 0 432L0 80C0 62.6 9.4 46.6 24.5 38.1z"/>
                                </svg>
                            </button>
                            <input type="range" id="progressBar" class="progress-bar" value="0" step="1">
                            <span id="currentTime" class="time">0:00</span> / <span id="duration" class="time">0:00</span>                           
                        </div>
                    </div>
                `;

                // Access the audio element and controls
                const audio = message.querySelector('audio');
                const audioPlayer = message.querySelector('.audio-player'); // Get the parent div

                // Apply background color to the parent div
                audioPlayer.style.backgroundColor = settingsData?.chatSetting?.userMessageColor;
                const playPauseBtn = message.querySelector('#playPauseBtn');
                const svgPath = playPauseBtn.querySelector('svg path');
                const progressBar = message.querySelector('#progressBar');
                const currentTimeDisplay = message.querySelector('#currentTime');
                const durationDisplay = message.querySelector('#duration');
                playPauseBtn.style.fill = settingsData?.chatSetting?.userMessageTextColor;
                svgPath.style.fill = settingsData?.chatSetting?.userMessageTextColor;
                currentTimeDisplay.style.color = settingsData?.chatSetting?.userMessageTextColor;
                durationDisplay.style.color = settingsData?.chatSetting?.userMessageTextColor;

                // Play/Pause functionality
                playPauseBtn.addEventListener('click', () => {
                    if (audio.paused) {
                        audio.play();
                        playPauseBtn.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 320 512"><path d="M48 80c-17.7 0-32 14.3-32 32l0 288c0 17.7 14.3 32 32 32l32 0c17.7 0 32-14.3 32-32l0-288c0-17.7-14.3-32-32-32L48 80zM0 112C0 85.5 21.5 64 48 64l32 0c26.5 0 48 21.5 48 48l0 288c0 26.5-21.5 48-48 48l-32 0c-26.5 0-48-21.5-48-48L0 112zM240 80c-17.7 0-32 14.3-32 32l0 288c0 17.7 14.3 32 32 32l32 0c17.7 0 32-14.3 32-32l0-288c0-17.7-14.3-32-32-32l-32 0zm-48 32c0-26.5 21.5-48 48-48l32 0c26.5 0 48 21.5 48 48l0 288c0 26.5-21.5 48-48 48l-32 0c-26.5 0-48-21.5-48-48l0-288z"/></svg>`;
                    } else {
                        audio.pause();
                        playPauseBtn.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 384 512"><path d="M64.7 52.7c-9.9-6-22.2-6.3-32.3-.6S16 68.4 16 80l0 352c0 11.6 6.3 22.2 16.3 27.9s22.5 5.4 32.3-.6l288-176c9.5-5.8 15.3-16.2 15.3-27.3s-5.8-21.5-15.3-27.3l-288-176zM24.5 38.1C39.7 29.6 58.2 30 73 39L361 215c14.3 8.7 23 24.2 23 41s-8.7 32.2-23 41L73 473c-14.8 9.1-33.4 9.4-48.5 .9S0 449.4 0 432L0 80C0 62.6 9.4 46.6 24.5 38.1z"/></svg>`;
                    }
                });

                // Update Progress Bar and Time
                audio.addEventListener('timeupdate', () => {
                    const currentTime = audio.currentTime;
                    const duration = audio.duration;
                    progressBar.value = (currentTime / duration) * 100;

                    const formatTime = (time) => {
                        const minutes = Math.floor(time / 60);
                        const seconds = Math.floor(time % 60);
                        return `${minutes}:${seconds < 10 ? '0' + seconds : seconds}`;
                    };

                    currentTimeDisplay.textContent = formatTime(currentTime);
                    durationDisplay.textContent = formatTime(duration);
                });

                // Update Audio Time with Progress Bar
                progressBar.addEventListener('input', (event) => {
                    const value = event.target.value;
                    const duration = audio.duration;
                    audio.currentTime = (value / 100) * duration;
                });

                // Reset the play button when the audio ends
                audio.addEventListener('ended', () => {
                    playPauseBtn.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 384 512"><path d="M64.7 52.7c-9.9-6-22.2-6.3-32.3-.6S16 68.4 16 80l0 352c0 11.6 6.3 22.2 16.3 27.9s22.5 5.4 32.3-.6l288-176c9.5-5.8 15.3-16.2 15.3-27.3s-5.8-21.5-15.3-27.3l-288-176zM24.5 38.1C39.7 29.6 58.2 30 73 39L361 215c14.3 8.7 23 24.2 23 41s-8.7 32.2-23 41L73 473c-14.8 9.1-33.4 9.4-48.5 .9S0 449.4 0 432L0 80C0 62.6 9.4 46.6 24.5 38.1z"/></svg>`;
                });

                console.log("Custom audio player created successfully:", audio);
            } else {
                message.textContent = "خطا در دریافت فایل صوتی.";
            }
        } catch (error) {
            console.error("خطا در پردازش فایل صوتی:", error);
            message.textContent = "مشکلی پیش آمده است. لطفاً دوباره تلاش کنید.";
        }
    } else {
        message.textContent = `${UserMessage}`;
    }
}


function UserChat(UserMessage, messageTime = new Date(), messageid, docid, DeliveryStatus) {

    var messageContainer = document.createElement('div');
    messageContainer.classList.add('message-container');
    var timestamp = document.createElement('div');
    timestamp.classList.add('message-timestamp');
    timestamp.setAttribute('data-time', messageTime);

    if (lang === 'fa' || lang === 'fa-IR') {
        timestamp.innerText = timeAgo(messageTime);
    } else {
        timestamp.innerText = timeAgo_EN(messageTime);
    }


    var message = document.createElement('div');
    message.classList.add('user-message');
    message.style.backgroundColor = settingsData?.chatSetting?.userMessageColor;
    message.style.color = settingsData?.chatSetting?.userMessageTextColor;

    let isVoiceMessage = false;

    if (docid) {
        try {
            const mediauri = ConvertToURL(docid);
            console.log("Generated URL:", mediauri);

            if (mediauri) {
                showaudio(docid, message, UserMessage);
                isVoiceMessage = true;
                message.style.backgroundColor = 'white';
            } else {
                message.textContent = "خطا در دریافت فایل صوتی.";
            }


            const audio = message.querySelector('audio');
            if (audio) {
                console.log("Audio element created successfully:", audio);
            } else {
                console.error("Audio element could not be created.");
            }
        } catch (error) {
            console.error("خطا در پردازش فایل صوتی:", error);
            message.textContent = "مشکلی پیش آمده است. لطفاً دوباره تلاش کنید.";
        }
    } else {
        message.innerHTML = `${UserMessage.replace(/\n/g, '<br>')}`;
    }


    var TimestampSatusIcon = document.createElement('div');
    TimestampSatusIcon.classList.add('timestamp-statusicon');
    var StatusIcon = document.createElement('div');
    StatusIcon.classList.add('status-icon');

    StatusIcon.innerHTML = (DeliveryStatus === 1)
        ? `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path d="M337 81c9.4-9.4 9.4-24.6 0-33.9s-24.6-9.4-33.9 0l-143 143L97 127c-9.4-9.4-24.6-9.4-33.9 0s-9.4 24.6 0 33.9l80 80c9.4 9.4 24.6 9.4 33.9 0L337 81zM441 201c9.4-9.4 9.4-24.6 0-33.9s24.6-9.4-33.9 0l-247 247L41 295c-9.4-9.4-24.6-9.4-33.9 0s-9.4 24.6 0 33.9L143 465c9.4 9.4 24.6 9.4 33.9 0L441 201z"/></svg>`
        : `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path d="M441 103c9.4 9.4 9.4 24.6 0 33.9L177 401c-9.4 9.4-24.6 9.4-33.9 0L7 265c-9.4-9.4-9.4-24.6 0-33.9s24.6-9.4 33.9 0l119 119L407 103c9.4-9.4 24.6-9.4 33.9 0z"/></svg>`;


    TimestampSatusIcon.appendChild(StatusIcon);
    TimestampSatusIcon.appendChild(timestamp);
    messageContainer.appendChild(message);
    messageContainer.appendChild(TimestampSatusIcon);
    chatWrapper.appendChild(messageContainer);
    chatWrapper.scrollTop = chatWrapper.scrollHeight;

    updateLastMessageMargin(true, isVoiceMessage);
    scrollToBottom();
   
}



// function UserChat(UserMessage, messageTime = new Date(), messageid, docid, DeliveryStatus) {


//     var messageContainer = document.createElement('div');
//     messageContainer.classList.add('message-container');
//     var timestamp = document.createElement('div');
//     timestamp.classList.add('message-timestamp');
//     timestamp.setAttribute('data-time', messageTime);

//     if (lang === 'fa' || lang === 'fa-IR') {
//         timestamp.innerText = timeAgo(messageTime);
//     } else {
//         timestamp.innerText = timeAgo_EN(messageTime);

//     }


//     // Create the message content
//     var message = document.createElement('div');
//     message.classList.add('user-message');
//     message.style.backgroundColor = settingsData?.chatSetting?.userMessageColor;
//     message.style.color = settingsData?.chatSetting?.userMessageTextColor;
//     let isVoiceMessage = false;


//     if (docid) {
//         try {
//             const mediauri = ConvertToURL(docid);
//             console.log("Generated URL:", mediauri);

//             if (mediauri) {
//                 showaudio(docid, message, UserMessage);
//                 isVoiceMessage = true;
//                 message.style.backgroundColor = 'white';
//             } else {
//                 message.textContent = "خطا در دریافت فایل صوتی.";
//             }

//             // بررسی دستی تگ audio
//             const audio = message.querySelector('audio');
//             if (audio) {
//                 console.log("Audio element created successfully:", audio);
//             } else {
//                 console.error("Audio element could not be created.");
//             }
//         } catch (error) {
//             console.error("خطا در پردازش فایل صوتی:", error);
//             message.textContent = "مشکلی پیش آمده است. لطفاً دوباره تلاش کنید.";
//         }
//     } else {
//         message.innerHTML = `${UserMessage.replace(/\n/g, '<br>')}`;
//     }


//     // Create the status icon container
//     var TimestampSatusIcon = document.createElement('div');
//     TimestampSatusIcon.classList.add('timestamp-statusicon');
//     var StatusIcon = document.createElement('div');
//     StatusIcon.classList.add('status-icon');


//     StatusIcon.innerHTML = (DeliveryStatus === 1)
//         ? `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path d="M337 81c9.4-9.4 9.4-24.6 0-33.9s-24.6-9.4-33.9 0l-143 143L97 127c-9.4-9.4-24.6-9.4-33.9 0s-9.4 24.6 0 33.9l80 80c9.4 9.4 24.6 9.4 33.9 0L337 81zM441 201c9.4-9.4 9.4-24.6 0-33.9s24.6-9.4-33.9 0l-247 247L41 295c-9.4-9.4-24.6-9.4-33.9 0s-9.4 24.6 0 33.9L143 465c9.4 9.4 24.6 9.4 33.9 0L441 201z"/></svg>`
//         : `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><path d="M441 103c9.4 9.4 9.4 24.6 0 33.9L177 401c-9.4 9.4-24.6 9.4-33.9 0L7 265c-9.4-9.4-9.4-24.6 0-33.9s24.6-9.4 33.9 0l119 119L407 103c9.4-9.4 24.6-9.4 33.9 0z"/></svg>`;




//     // Timeout for checking delivery status
//     // setTimeout(() => {
//     //     if (navigator.onLine) {
//     //         DeliveryStatus = 1;

//     //     } else {
//     //         console.warn('Cannot update status because the internet is offline.');
//     //         DeliveryStatus = 0;
//     //         ShowTypingIndicator('off');
//     //         DisableSendMsg(false);
//     //         localStorage.setItem('lastFailedMessage', JSON.stringify({ UserMessage, messageTime }));
//     //         StatusIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path d="M417.3 360.2c3.2-5 8.7-8.2 14.7-8.2c12.3 0 20.3 12.8 13.7 23.2C406 438.2 335.9 480 256 480C132.3 480 32 379.7 32 256S132.3 32 256 32c56.1 0 107.4 20.6 146.7 54.7L433.4 56c5.1-5.1 12.1-8 19.3-8C467.8 48 480 60.2 480 75.3L480 176c0 8.8-7.2 16-16 16l-100.7 0c-15.1 0-27.3-12.2-27.3-27.3c0-7.2 2.9-14.2 8-19.3l36-36C346.5 81.1 303.3 64 256 64C150 64 64 150 64 256s86 192 192 192c67.6 0 127.1-35 161.3-87.8zM448 86.6L374.6 160l73.4 0 0-73.4z"/></svg>`;
//     //         resendMessage();
//     //     }
//     // }, 10000);


//     // Send the message to the server
//     // RSendMessage(UserMessage);

//     // Append elements
//     TimestampSatusIcon.appendChild(StatusIcon);
//     TimestampSatusIcon.appendChild(timestamp);
//     messageContainer.appendChild(message);
//     messageContainer.appendChild(TimestampSatusIcon);
//     chatWrapper.appendChild(messageContainer);
//     chatWrapper.scrollTop = chatWrapper.scrollHeight;
//     updateLastMessageMargin(true, isVoiceMessage);
//     scrollToBottom();
// }



connection.on("ChangeMessageDelivery", (message) => {
    console.log('ID : ' + message.messageId + ' status: ' + message.status);
    const StatusIcon = document.querySelectorAll('.status-icon');
    const lastStatusIcon = StatusIcon[StatusIcon.length - 1];
    if (message.status === 0) {
        console.log('ارسال شد');
        lastStatusIcon.innerHTML = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><!--!Font Awesome Pro 6.7.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path class="fa-secondary" opacity=".4" d=""/><path class="fa-primary" d="M443.3 100.7c6.2 6.2 6.2 16.4 0 22.6l-272 272c-6.2 6.2-16.4 6.2-22.6 0l-144-144c-6.2-6.2-6.2-16.4 0-22.6s16.4-6.2 22.6 0L160 361.4 420.7 100.7c6.2-6.2 16.4-6.2 22.6 0z"/></svg>';
    } else if (message.status === 1) {
        console.log('مشاهده شد');
        lastStatusIcon.innerHTML = '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><!--!Font Awesome Pro 6.7.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path d="M331.3 75.3c6.2-6.2 6.2-16.4 0-22.6s-16.4-6.2-22.6 0L160 201.4 91.3 132.7c-6.2-6.2-16.4-6.2-22.6 0s-6.2 16.4 0 22.6l80 80c6.2 6.2 16.4 6.2 22.6 0l160-160zm112 112c6.2-6.2 6.2-16.4 0-22.6s-16.4-6.2-22.6 0L160 425.4 27.3 292.7c-6.2-6.2-16.4-6.2-22.6 0s-6.2 16.4 0 22.6l144 144c6.2 6.2 16.4 6.2 22.6 0l272-272z"/></svg>';
    } else if (message.status === 2) {
        console.log('درحال جواب');
    }
});



async function showBotAudio(docId, botMessage, ReceviedMessage) {

    if (docId) {
        try {
            const mediauri = await ConvertToURL(docId);
            if (mediauri) {
                // Custom audio player structure
                botMessage.innerHTML = `
                    <div class="audio-player">
                        <audio id="audio" controls>
                            <source src="${mediauri}" type="audio/mp3">
                        </audio>
                        <div class="controls">
                            <button id="playPauseBtn" class="play-pause-btn">
                                <!-- Play icon (initial state) -->
                                <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 384 512">
                                    <path d="M64.7 52.7c-9.9-6-22.2-6.3-32.3-.6S16 68.4 16 80l0 352c0 11.6 6.3 22.2 16.3 27.9s22.5 5.4 32.3-.6l288-176c9.5-5.8 15.3-16.2 15.3-27.3s-5.8-21.5-15.3-27.3l-288-176zM24.5 38.1C39.7 29.6 58.2 30 73 39L361 215c14.3 8.7 23 24.2 23 41s-8.7 32.2-23 41L73 473c-14.8 9.1-33.4 9.4-48.5 .9S0 449.4 0 432L0 80C0 62.6 9.4 46.6 24.5 38.1z"/>
                                </svg>
                            </button>
                            <input type="range" id="progressBar" class="progress-bar" value="0" step="1">
                            <span id="currentTime" class="time">0:00</span> / <span id="duration" class="time">0:00</span>                           
                        </div>
                    </div>
                `;

                // Access the audio element and controls
                const audio = botMessage.querySelector('audio');
                const audioPlayer = botMessage.querySelector('.audio-player'); // Get the parent div

                // Apply background color to the parent div
                audioPlayer.style.backgroundColor = settingsData?.chatSetting?.responseMessageColor;
                const playPauseBtn = botMessage.querySelector('#playPauseBtn');
                const svgPath = playPauseBtn.querySelector('svg path');
                const progressBar = botMessage.querySelector('#progressBar');
                const currentTimeDisplay = botMessage.querySelector('#currentTime');
                const durationDisplay = botMessage.querySelector('#duration');
                playPauseBtn.style.fill = settingsData?.chatSetting?.responseMessageTextColor;
                svgPath.style.fill = settingsData?.chatSetting?.responseMessageTextColor;
                currentTimeDisplay.style.color = settingsData?.chatSetting?.responseMessageTextColor;
                durationDisplay.style.color = settingsData?.chatSetting?.responseMessageTextColor;

                // Play/Pause functionality
                playPauseBtn.addEventListener('click', () => {
                    if (audio.paused) {
                        audio.play();
                        playPauseBtn.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 320 512"><path d="M48 80c-17.7 0-32 14.3-32 32l0 288c0 17.7 14.3 32 32 32l32 0c17.7 0 32-14.3 32-32l0-288c0-17.7-14.3-32-32-32L48 80zM0 112C0 85.5 21.5 64 48 64l32 0c26.5 0 48 21.5 48 48l0 288c0 26.5-21.5 48-48 48l-32 0c-26.5 0-48-21.5-48-48L0 112zM240 80c-17.7 0-32 14.3-32 32l0 288c0 17.7 14.3 32 32 32l32 0c17.7 0 32-14.3 32-32l0-288c0-17.7-14.3-32-32-32l-32 0zm-48 32c0-26.5 21.5-48 48-48l32 0c26.5 0 48 21.5 48 48l0 288c0 26.5-21.5 48-48 48l-32 0c-26.5 0-48-21.5-48-48l0-288z"/></svg>`;
                    } else {
                        audio.pause();
                        playPauseBtn.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 384 512"><path d="M64.7 52.7c-9.9-6-22.2-6.3-32.3-.6S16 68.4 16 80l0 352c0 11.6 6.3 22.2 16.3 27.9s22.5 5.4 32.3-.6l288-176c9.5-5.8 15.3-16.2 15.3-27.3s-5.8-21.5-15.3-27.3l-288-176zM24.5 38.1C39.7 29.6 58.2 30 73 39L361 215c14.3 8.7 23 24.2 23 41s-8.7 32.2-23 41L73 473c-14.8 9.1-33.4 9.4-48.5 .9S0 449.4 0 432L0 80C0 62.6 9.4 46.6 24.5 38.1z"/></svg>`;
                    }
                });

                // Update Progress Bar and Time
                audio.addEventListener('timeupdate', () => {
                    const currentTime = audio.currentTime;
                    const duration = audio.duration;
                    progressBar.value = (currentTime / duration) * 100;

                    const formatTime = (time) => {
                        const minutes = Math.floor(time / 60);
                        const seconds = Math.floor(time % 60);
                        return `${minutes}:${seconds < 10 ? '0' + seconds : seconds}`;
                    };

                    currentTimeDisplay.textContent = formatTime(currentTime);
                    durationDisplay.textContent = formatTime(duration);
                });

                // Update Audio Time with Progress Bar
                progressBar.addEventListener('input', (event) => {
                    const value = event.target.value;
                    const duration = audio.duration;
                    audio.currentTime = (value / 100) * duration;
                });

                // Reset the play button when the audio ends
                audio.addEventListener('ended', () => {
                    playPauseBtn.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 384 512"><path d="M64.7 52.7c-9.9-6-22.2-6.3-32.3-.6S16 68.4 16 80l0 352c0 11.6 6.3 22.2 16.3 27.9s22.5 5.4 32.3-.6l288-176c9.5-5.8 15.3-16.2 15.3-27.3s-5.8-21.5-15.3-27.3l-288-176zM24.5 38.1C39.7 29.6 58.2 30 73 39L361 215c14.3 8.7 23 24.2 23 41s-8.7 32.2-23 41L73 473c-14.8 9.1-33.4 9.4-48.5 .9S0 449.4 0 432L0 80C0 62.6 9.4 46.6 24.5 38.1z"/></svg>`;
                });

                console.log("Custom audio player created successfully:", audio);
            } else {
                botMessage.textContent = "خطا در دریافت فایل صوتی.";
            }
        } catch (error) {
            console.error("خطا در پردازش فایل صوتی:", error);
            botMessage.textContent = "مشکلی پیش آمده است. لطفاً دوباره تلاش کنید.";
        }
    } else {
        botMessage.textContent = `${ReceviedMessage}`;
    }
}


function ChatAgent(ReceviedMessage, messageTime = new Date(), messageId, status, role, docId ) {
	docId = docId ?? null;
    var messageContainer = document.createElement('div');
    messageContainer.classList.add('bot-message-container'); // For bot message container
    var botMessage = document.createElement('div');

    botMessage.classList.add('bot-message');
    botMessage.style.backgroundColor = settingsData?.chatSetting?.responseMessageColor;
    botMessage.style.color = settingsData?.chatSetting?.responseMessageTextColor;


    var timestamp = document.createElement('div');
    timestamp.classList.add('message-timestamp');
    timestamp.setAttribute('data-time', messageTime);
    if (lang === 'fa' || lang === 'fa-IR') {
        timestamp.innerText = timeAgo(messageTime);
    } else {
        timestamp.innerText = timeAgo_EN(messageTime);

    }

    let isVoiceMessage = false;
    // Append bot message to the message container
    messageContainer.appendChild(botMessage);

    if (docId) {
        try {
            const mediauri = ConvertToURL(docId);
            console.log("Generated URL:", mediauri);

            if (mediauri) {
                showBotAudio(docId, botMessage, ReceviedMessage);
                isVoiceMessage = true;
                botMessage.style.backgroundColor = 'white';
            } else {
                botMessage.textContent = "خطا در دریافت فایل صوتی.";
            }

            // بررسی دستی تگ audio
            const audio = botMessage.querySelector('audio');
            if (audio) {
                console.log("Audio element created successfully:", audio);
            } else {
                console.error("Audio element could not be created.");
            }
        } catch (error) {
            console.error("خطا در پردازش فایل صوتی:", error);
            botMessage.textContent = "مشکلی پیش آمده است. لطفاً دوباره تلاش کنید.";
        }
    } else {
        botMessage.textContent = `${ReceviedMessage}`;

    }


    if (botMessage.textContent === "در حال حاضر تاویا در دسترس نمی‌باشد") {
        botMessage.classList.add('failure-msg');
    }


    // Container for the timestamp and reaction icons
    var botReactionAndTimestampContainer = document.createElement('div');
    botReactionAndTimestampContainer.classList.add('bot-reaction-timestamp-container');

    let isFirstMessage = chatWrapper.querySelectorAll('.bot-message-container').length === 0;

    // Add reaction container with messageId if not the first message
    if (role !== 2 && !isFirstMessage) {
        var reactionContainer = CreateReactionChat(status, messageId); // Pass messageId here
        botReactionAndTimestampContainer.appendChild(reactionContainer);
    }

    // Append the timestamp to the same container
    botReactionAndTimestampContainer.appendChild(timestamp);

    // Append the combined container to the message container
    messageContainer.appendChild(botReactionAndTimestampContainer);

    chatWrapper.appendChild(messageContainer);
    chatWrapper.scrollTop = chatWrapper.scrollHeight;
    startTimestampUpdater();
    updateLastMessageMargin(true, isVoiceMessage);
    scrollToBottom();
}


function CreateReactionChat(opinionStatus, messageId) {
    var likeIcon = document.createElement('div');
    var dislikeIcon = document.createElement('div');
    likeIcon.classList.add('reaction-icon');
    dislikeIcon.classList.add('reaction-icon');

    function updateIcons() {
        if (opinionStatus === 0) {
            likeIcon.classList.add('active-like');
            dislikeIcon.classList.remove('active-dislike');
            likeIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><!--!Font Awesome Free 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free Copyright 2024 Fonticons, Inc.--><path d="M313.4 32.9c26 5.2 42.9 30.5 37.7 56.5l-2.3 11.4c-5.3 26.7-15.1 52.1-28.8 75.2l144 0c26.5 0 48 21.5 48 48c0 18.5-10.5 34.6-25.9 42.6C497 275.4 504 288.9 504 304c0 23.4-16.8 42.9-38.9 47.1c4.4 7.3 6.9 15.8 6.9 24.9c0 21.3-13.9 39.4-33.1 45.6c.7 3.3 1.1 6.8 1.1 10.4c0 26.5-21.5 48-48 48l-97.5 0c-19 0-37.5-5.6-53.3-16.1l-38.5-25.7C176 420.4 160 390.4 160 358.3l0-38.3 0-48 0-24.9c0-29.2 13.3-56.7 36-75l7.4-5.9c26.5-21.2 44.6-51 51.2-84.2l2.3-11.4c5.2-26 30.5-42.9 56.5-37.7zM32 192l64 0c17.7 0 32 14.3 32 32l0 224c0 17.7-14.3 32-32 32l-64 0c-17.7 0-32-14.3-32-32L0 224c0-17.7 14.3-32 32-32z"/></svg>`;
            dislikeIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><!--!Font Awesome Pro 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path d="M288.8 430.3c3.5 12.8 16.7 20.3 29.5 16.8s20.3-16.7 16.8-29.5l-4.5-16.4c-5.5-20.2-13.9-39.3-24.7-56.9c-3.1-4.9-3.2-11.1-.4-16.2s8.2-8.2 14-8.2L448 320c17.7 0 32-14.3 32-32c0-11.3-5.9-21.3-14.8-27c-7.2-4.6-9.5-13.9-5.3-21.3c2.6-4.6 4.1-10 4.1-15.7c0-12.4-7-23.1-17.3-28.5c-4.2-2.2-7.3-6.1-8.3-10.8s.1-9.5 3-13.2c4.2-5.4 6.7-12.2 6.7-19.5c0-14.2-9.2-26.3-22.1-30.4c-7.8-2.5-12.4-10.6-10.7-18.6c.5-2.2 .7-4.5 .7-6.9c0-17.7-14.3-32-32-32l-89.5 0c-15.8 0-31.2 4.7-44.4 13.4l-38.5 25.7c-9 6-16.6 13.7-22.4 22.6c-4.9 7.4-14.8 9.4-22.2 4.6s-9.4-14.8-4.6-22.2c8.1-12.3 18.7-23.1 31.4-31.6l38.5-25.7c18.4-12.3 40-18.8 62.1-18.8L384 32c35.3 0 64 28.7 64 64l0 .6c19.1 11.1 32 31.7 32 55.4c0 8.7-1.8 17.1-4.9 24.7C487.9 188.4 496 205.2 496 224c0 6.5-1 12.8-2.8 18.7C504.8 254.3 512 270.3 512 288c0 35.3-28.7 64-64 64l-101.6 0c6.2 13.1 11.3 26.7 15.1 40.9l4.5 16.4c8.1 29.8-9.5 60.6-39.3 68.8s-60.6-9.5-68.8-39.3l-4.5-16.4c-8.9-32.6-29.6-60.8-58.2-79l-3.1-2 8.2-12.9-8.2 12.9c-11.8-7.5-21.7-17.1-29.5-28.2c-5.1-7.2-3.3-17.2 4-22.3s17.2-3.3 22.3 4c5.4 7.7 12.2 14.4 20.4 19.5l3.1 2c35.3 22.4 60.9 57.2 71.9 97.5l4.5 16.4zM32 352l64 0 0-224-64 0 0 224zM0 352L0 128c0-17.7 14.3-32 32-32l64 0c17.7 0 32 14.3 32 32l0 224c0 17.7-14.3 32-32 32l-64 0c-17.7 0-32-14.3-32-32z"/></svg>`;
        } else if (opinionStatus === 1) {
            dislikeIcon.classList.add('active-dislike');
            likeIcon.classList.remove('active-like');
            dislikeIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><!--!Font Awesome Free 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free Copyright 2024 Fonticons, Inc.--><path d="M313.4 479.1c26-5.2 42.9-30.5 37.7-56.5l-2.3-11.4c-5.3-26.7-15.1-52.1-28.8-75.2l144 0c26.5 0 48-21.5 48-48c0-18.5-10.5-34.6-25.9-42.6C497 236.6 504 223.1 504 208c0-23.4-16.8-42.9-38.9-47.1c4.4-7.3 6.9-15.8 6.9-24.9c0-21.3-13.9-39.4-33.1-45.6c.7-3.3 1.1-6.8 1.1-10.4c0-26.5-21.5-48-48-48l-97.5 0c-19 0-37.5 5.6-53.3 16.1L202.7 73.8C176 91.6 160 121.6 160 153.7l0 38.3 0 48 0 24.9c0 29.2 13.3 56.7 36 75l7.4 5.9c26.5 21.2 44.6 51 51.2 84.2l2.3 11.4c5.2 26 30.5 42.9 56.5 37.7zM32 384l64 0c17.7 0 32-14.3 32-32l0-224c0-17.7-14.3-32-32-32L32 96C14.3 96 0 110.3 0 128L0 352c0 17.7 14.3 32 32 32z"/></svg>`;
            likeIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><!--!Font Awesome Pro 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path d="M288.8 81.7c3.5-12.8 16.7-20.3 29.5-16.8s20.3 16.7 16.8 29.5l-4.5 16.4c-5.5 20.2-13.9 39.3-24.7 56.9c-3.1 4.9-3.2 11.1-.4 16.2s8.2 8.2 14 8.2L448 192c17.7 0 32 14.3 32 32c0 11.3-5.9 21.3-14.8 27c-7.2 4.6-9.5 13.9-5.3 21.3c2.6 4.6 4.1 10 4.1 15.7c0 12.4-7 23.1-17.3 28.5c-4.2 2.2-7.3 6.1-8.3 10.8s.1 9.5 3 13.2c4.2 5.4 6.7 12.2 6.7 19.5c0 14.2-9.2 26.3-22.1 30.4c-7.8 2.5-12.4 10.6-10.7 18.6c.5 2.2 .7 4.5 .7 6.9c0 17.7-14.3 32-32 32l-89.5 0c-15.8 0-31.2-4.7-44.4-13.4l-38.5-25.7c-9-6-16.6-13.7-22.4-22.6c-4.9-7.4-14.8-9.4-22.2-4.6s-9.4 14.8-4.6 22.2c8.1 12.3 18.7 23.1 31.4 31.6l38.5 25.7c18.4 12.3 40 18.8 62.1 18.8l89.5 0c35.3 0 64-28.7 64-64l0-.6c19.1-11.1 32-31.7 32-55.4c0-8.7-1.8-17.1-4.9-24.7C487.9 323.6 496 306.8 496 288c0-6.5-1-12.8-2.8-18.7C504.8 257.7 512 241.7 512 224c0-35.3-28.7-64-64-64l-101.6 0c6.2-13.1 11.3-26.7 15.1-40.9l4.5-16.4c8.1-29.8-9.5-60.6-39.3-68.8s-60.6 9.5-68.8 39.3l-4.5 16.4c-8.9 32.6-29.6 60.8-58.2 79l-3.1 2c-11.8 7.5-21.7 17.1-29.5 28.2c-5.1 7.2-3.3 17.2 4 22.3s17.2 3.3 22.3-4c5.4-7.7 12.2-14.4 20.4-19.5l3.1-2c35.3-22.4 60.9-57.2 71.9-97.5l4.5-16.4zM32 224l64 0 0 224-64 0 0-224zM0 224L0 448c0 17.7 14.3 32 32 32l64 0c17.7 0 32-14.3 32-32l0-224c0-17.7-14.3-32-32-32l-64 0c-17.7 0-32 14.3-32 32z"/></svg>`;
        } else if (opinionStatus === 2) {
            dislikeIcon.innerHTML = '';
            likeIcon.innerHTML = '';
        }
        else {
            likeIcon.classList.remove('active-like');
            dislikeIcon.classList.remove('active-dislike');
            likeIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><!--!Font Awesome Pro 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path d="M288.8 81.7c3.5-12.8 16.7-20.3 29.5-16.8s20.3 16.7 16.8 29.5l-4.5 16.4c-5.5 20.2-13.9 39.3-24.7 56.9c-3.1 4.9-3.2 11.1-.4 16.2s8.2 8.2 14 8.2L448 192c17.7 0 32 14.3 32 32c0 11.3-5.9 21.3-14.8 27c-7.2 4.6-9.5 13.9-5.3 21.3c2.6 4.6 4.1 10 4.1 15.7c0 12.4-7 23.1-17.3 28.5c-4.2 2.2-7.3 6.1-8.3 10.8s.1 9.5 3 13.2c4.2 5.4 6.7 12.2 6.7 19.5c0 14.2-9.2 26.3-22.1 30.4c-7.8 2.5-12.4 10.6-10.7 18.6c.5 2.2 .7 4.5 .7 6.9c0 17.7-14.3 32-32 32l-89.5 0c-15.8 0-31.2-4.7-44.4-13.4l-38.5-25.7c-9-6-16.6-13.7-22.4-22.6c-4.9-7.4-14.8-9.4-22.2-4.6s-9.4 14.8-4.6 22.2c8.1 12.3 18.7 23.1 31.4 31.6l38.5 25.7c18.4 12.3 40 18.8 62.1 18.8l89.5 0c35.3 0 64-28.7 64-64l0-.6c19.1-11.1 32-31.7 32-55.4c0-8.7-1.8-17.1-4.9-24.7C487.9 323.6 496 306.8 496 288c0-6.5-1-12.8-2.8-18.7C504.8 257.7 512 241.7 512 224c0-35.3-28.7-64-64-64l-101.6 0c6.2-13.1 11.3-26.7 15.1-40.9l4.5-16.4c8.1-29.8-9.5-60.6-39.3-68.8s-60.6 9.5-68.8 39.3l-4.5 16.4c-8.9 32.6-29.6 60.8-58.2 79l-3.1 2c-11.8 7.5-21.7 17.1-29.5 28.2c-5.1 7.2-3.3 17.2 4 22.3s17.2 3.3 22.3-4c5.4-7.7 12.2-14.4 20.4-19.5l3.1-2c35.3-22.4 60.9-57.2 71.9-97.5l4.5-16.4zM32 224l64 0 0 224-64 0 0-224zM0 224L0 448c0 17.7 14.3 32 32 32l64 0c17.7 0 32-14.3 32-32l0-224c0-17.7-14.3-32-32-32l-64 0c-17.7 0-32 14.3-32 32z"/></svg>`;
            dislikeIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><!--!Font Awesome Pro 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path d="M288.8 430.3c3.5 12.8 16.7 20.3 29.5 16.8s20.3-16.7 16.8-29.5l-4.5-16.4c-5.5-20.2-13.9-39.3-24.7-56.9c-3.1-4.9-3.2-11.1-.4-16.2s8.2-8.2 14-8.2L448 320c17.7 0 32-14.3 32-32c0-11.3-5.9-21.3-14.8-27c-7.2-4.6-9.5-13.9-5.3-21.3c2.6-4.6 4.1-10 4.1-15.7c0-12.4-7-23.1-17.3-28.5c-4.2-2.2-7.3-6.1-8.3-10.8s.1-9.5 3-13.2c4.2-5.4 6.7-12.2 6.7-19.5c0-14.2-9.2-26.3-22.1-30.4c-7.8-2.5-12.4-10.6-10.7-18.6c.5-2.2 .7-4.5 .7-6.9c0-17.7-14.3-32-32-32l-89.5 0c-15.8 0-31.2 4.7-44.4 13.4l-38.5 25.7c-9 6-16.6 13.7-22.4 22.6c-4.9 7.4-14.8 9.4-22.2 4.6s-9.4-14.8-4.6-22.2c8.1-12.3 18.7-23.1 31.4-31.6l38.5-25.7c18.4-12.3 40-18.8 62.1-18.8L384 32c35.3 0 64 28.7 64 64l0 .6c19.1 11.1 32 31.7 32 55.4c0 8.7-1.8 17.1-4.9 24.7C487.9 188.4 496 205.2 496 224c0 6.5-1 12.8-2.8 18.7C504.8 254.3 512 270.3 512 288c0 35.3-28.7 64-64 64l-101.6 0c6.2 13.1 11.3 26.7 15.1 40.9l4.5 16.4c8.1 29.8-9.5 60.6-39.3 68.8s-60.6-9.5-68.8-39.3l-4.5-16.4c-8.9-32.6-29.6-60.8-58.2-79l-3.1-2 8.2-12.9-8.2 12.9c-11.8-7.5-21.7-17.1-29.5-28.2c-5.1-7.2-3.3-17.2 4-22.3s17.2-3.3 22.3 4c5.4 7.7 12.2 14.4 20.4 19.5l3.1 2c35.3 22.4 60.9 57.2 71.9 97.5l4.5 16.4zM32 352l64 0 0-224-64 0 0 224zM0 352L0 128c0-17.7 14.3-32 32-32l64 0c17.7 0 32 14.3 32 32l0 224c0 17.7-14.3 32-32 32l-64 0c-17.7 0-32-14.3-32-32z"/></svg>`;
        }
    }

    // Initial icon update
    updateIcons();

    var reactionContainer = document.createElement('div');
    reactionContainer.classList.add('reaction-container');

    // رویداد کلیک برای آیکن لایک
    likeIcon.addEventListener('click', function () {
        // اگر در وضعیت لایک قبلاً فعال بوده باشد، عمل حذف را انجام دهید
        if (opinionStatus === 0) {
            RRemoveAction(messageId);
        }

        // غیرفعال کردن هر دو آیکن با اضافه کردن کلاس disabled
        likeIcon.classList.add('disabled');
        dislikeIcon.classList.add('disabled');

        // تغییر وضعیت لایک
        opinionStatus = opinionStatus === 0 ? null : 0;
        updateIcons();

        if (opinionStatus === 0) {
            RLike(messageId);
        }

        // پس از 5000 میلی‌ثانیه (5 ثانیه) هر دو آیکن دوباره فعال می‌شوند
        setTimeout(function () {
            likeIcon.classList.remove('disabled');
            dislikeIcon.classList.remove('disabled');
        }, 800);
    });

    // رویداد کلیک برای آیکن دیسلایک
    dislikeIcon.addEventListener('click', function () {
        // اگر در وضعیت دیسلایک قبلاً فعال بوده باشد، عمل حذف را انجام دهید
        if (opinionStatus === 1) {
            RRemoveAction(messageId);
        }

        // غیرفعال کردن هر دو آیکن با اضافه کردن کلاس disabled
        likeIcon.classList.add('disabled');
        dislikeIcon.classList.add('disabled');

        // تغییر وضعیت دیسلایک
        opinionStatus = opinionStatus === 1 ? null : 1;
        updateIcons();

        if (opinionStatus === 1) {
            RDisLike(messageId);
        }

        // پس از 5000 میلی‌ثانیه (5 ثانیه) هر دو آیکن دوباره فعال می‌شوند
        setTimeout(function () {
            likeIcon.classList.remove('disabled');
            dislikeIcon.classList.remove('disabled');
        }, 800);
    });



    reactionContainer.appendChild(likeIcon);
    reactionContainer.appendChild(dislikeIcon);

    return reactionContainer;
}



function timeAgo(time) {
    const now = new Date();
    const diffInSeconds = Math.floor((now - new Date(time)) / 1000); // اختلاف زمان به ثانیه

    if (diffInSeconds < 60) return "چند لحظه پیش";
    const diffInMinutes = Math.floor(diffInSeconds / 60);
    if (diffInMinutes < 60) return `${diffInMinutes} دقیقه پیش`;

    const diffInHours = Math.floor(diffInMinutes / 60);
    if (diffInHours < 24) return `${diffInHours} ساعت پیش`;

    const diffInDays = Math.floor(diffInHours / 24);
    return `${diffInDays} روز پیش`;
}



function timeAgo_EN(time) {
    const now = new Date();
    const diffInSeconds = Math.floor((now - new Date(time)) / 1000);

    if (diffInSeconds < 60) return "A few moments ago";
    const diffInMinutes = Math.floor(diffInSeconds / 60);
    if (diffInMinutes < 60) return `${diffInMinutes} minutes ago`;

    const diffInHours = Math.floor(diffInMinutes / 60);
    if (diffInHours < 24) return `${diffInHours} hours ago`;

    const diffInDays = Math.floor(diffInHours / 24);
    return `${diffInDays} days ago`;
}


function startTimestampUpdater() {
    setInterval(() => {
        const timestamps = document.querySelectorAll('.message-timestamp');
        timestamps.forEach((timestamp) => {
            const time = timestamp.getAttribute('data-time');
            if (lang === 'fa' || lang === 'fa-IR') {
                timestamp.innerText = timeAgo(time);
            } else {
                timestamp.innerText = timeAgo_EN(time);

            }


        });
    }, 60000);
}


async function createChatBox() {


    var chatBox = document.createElement('div');
    chatBox.classList.add('chatbox');
    // Create the body of the chatbox where the sections will go
    var chatBoxBody = document.createElement('div');
    chatBoxBody.classList.add('chatbox-body');

    if (settingsData?.buttonSetting?.chatButtonPositionId === 0) {
        chatBox.style.left = '20px';
    }


    // Create section 1 (Header or Controls) and append it
    chatBox.appendChild(createChatboxHeader());
    chatBox.appendChild(companySignMainAtag);
    chatBoxBody.appendChild(chatWrapper);



    // Create the sections once
    var section2 = createChatboxForm();
    var section3 = createChatboxMessageContainer();

    // Append both sections, but set section3 to hidden by default
    section3.style.display = 'none';
    chatBoxBody.appendChild(section2);
    chatBoxBody.appendChild(section3);

    chatBox.appendChild(chatBoxBody);
    const isActived = await isActive();

    if( isActived === 'false'){
        icon.style.display = 'none';
    }else {
        icon.style.display = 'flex';
    }
    
    let ConversationId = localStorage.getItem('ChatID');
    if (ConversationId) {
        const isExpired = await Expire(ConversationId);

    
        if (isExpired === 'true') {
            localStorage.removeItem('ChatID');
            // If expired, show section2 (default form view)
            section2.style.display = 'flex'; // Show section 2
            section3.style.display = 'none'; // Hide section 3
            chatWrapper.style.display = 'none';

        } else if (isExpired === 'false') {
            // If not expired, show section3 (chat view)
            section2.style.display = 'none'; // Hide section 2
            section3.style.display = 'flex'; // Show section 3
            chatWrapper.style.display = 'flex';

        } else {
            console.log(isExpired);
            console.log("An error occurred.");
            localStorage.removeItem('ChatID');
        }
    } else {
        // If no ConversationId, show section2 (default view)
        section2.style.display = 'flex'; // Show section 2
        section3.style.display = 'none'; // Hide section 3
        chatWrapper.style.display = 'none';
    }

    return chatBox;
}

var icon = document.createElement('div');
async function createChatIcon() {
    console.log("تابع createChatIcon اجرا شد");

    // گرفتن ChatID از localStorage
    let CID = localStorage.getItem('ChatID');
    await RStart();
    if (CID) {
        await RJoin(CID);
    }


    // ساختن دکمه چت و اضافه کردن کلاس‌ها
    var iconDiv = document.createElement('div');
    var ChatIconImg = document.createElement('img');
    ChatIconImg.setAttribute('id', 'ICON');
    icon.classList.add('chat-icon');
    icon.appendChild(ChatIconImg);
    iconDiv.appendChild(icon);
    console.log("دکمه چت ایجاد شد");

    // بررسی وجود icon و تنظیم تصویر
    let imageUrl;

    if (settingsData?.buttonSetting?.icon?.documentId && settingsData?.buttonSetting?.icon?.extension) {
        imageUrl = await ConvertToURL(settingsData.buttonSetting.icon.documentId, settingsData.buttonSetting.icon.extension);
    } else {
        imageUrl = 'https://learn.taavsys.com/chat/icons/headset-light2.png';

    }

    // تنظیم src تصویر دکمه چت
    ChatIconImg.src = imageUrl;


    if (settingsData?.buttonSetting?.chatButtonPositionId === 0) {
        icon.style.left = '0';
    }

    icon.style.backgroundColor = settingsData.buttonSetting.color; // Replace 'defaultColor' with a fallback color if needed


    icon.addEventListener('click', toggleChatBox);

    return iconDiv;
}

async function isActive() {
    const myHeaders = new Headers();
    myHeaders.append("tenantId", TenantId);

    const requestOptions = {
        method: "GET",
        headers: myHeaders,
        redirect: "follow"
    };

    try {
        const response = await fetch(BaseUri + "/api/v1/products/" + ProductId + "/is-active", requestOptions);
        const result = await response.text();
        return result;
    } catch (error) {
        console.error(error);
        throw error;
    }
}

async function Expire(Chat) {
    const myHeaders = new Headers();
    myHeaders.append("tenantId", TenantId);

    const requestOptions = {
        method: "GET",
        headers: myHeaders,
        redirect: "follow"
    };

    try {
        const response = await fetch(BaseUri + "/api/v1/conversations/" + localStorage.getItem('ChatID') + "/is-expired", requestOptions);
        const result = await response.text();
        return result;
    } catch (error) {
        console.error(error);
        throw error;
    }
}


async function Get_Items() {
    try {
        const myHeaders = new Headers();
        myHeaders.append("tenantId", TenantId);

        const requestOptions = {
            method: "GET",
            headers: myHeaders,
            redirect: "follow"
        };

        const chatID = localStorage.getItem('ChatID');

        const response = await fetch(BaseUri + "/api/v1/conversations/" + localStorage.getItem('ChatID') + "/chat-messages", requestOptions);

        // بررسی وضعیت پاسخ
        if (!response.ok) {
            throw new Error(`HTTP error! Status: ${response.status}`);
        }

        // Access the JSON data (this is the "result" you're asking about)
        const result = await response.json(); // تغییر به json برای پردازش بهتر داده‌ها


        // You can return the result if you need to access it outside this function
        return result;

    } catch (error) {
        console.error('Error fetching chat messages:', error);
    }

    return Get_Items;
}


async function ListingChat() {
    const result = await Get_Items();
    result.elements.sort((a, b) => new Date(a.time) - new Date(b.time));

    result.elements.forEach((element) => {
		var docId;
		if(element.messageContent.media){
			console.log('media IDDDDDDDDDDD: ' , element.messageContent.media.id );
			docId= element.messageContent.media.id;
		}else{
			docId= null;
		}
		
        console.log('Message DeliveryStatus:', element.deliveryStatusId, element.messageContent.text);
        if (element.role === 1) { // User message
            messageContainer = UserChat(element.messageContent.text, element.time, element.messageContent.id, docId, element.deliveryStatusId);
        } else { // Agent message
            messageContainer = ChatAgent(element.messageContent.text, element.time, element.messageContent.id, element.opinionStatus, element.role, docId );
        }
    });
}

ListingChat();




// Function to create the first section (header)

var innerDiv2Text = document.createElement('p');

function createChatboxHeader() {
    var Div1 = document.createElement('div');
    Div1.classList.add('chatbox-inside-section1');

    var innerDiv1 = document.createElement('div');
    innerDiv1.classList.add('inner-div-1');
    var innericondiv1 = document.createElement('div');
    innericondiv1.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" width="22" height="18" viewBox="0 0 22 18" fill="none">
<path d="M18.6667 9.33398H12C11.4696 9.33398 10.9609 9.5447 10.5858 9.91977C10.2107 10.2948 10 10.8036 10 11.334V15.334C10 15.8644 10.2107 16.3731 10.5858 16.7482C10.9609 17.1233 11.4696 17.334 12 17.334H18.6667C19.1971 17.334 19.7058 17.1233 20.0809 16.7482C20.4559 16.3731 20.6667 15.8644 20.6667 15.334V11.334C20.6667 10.8036 20.4559 10.2948 20.0809 9.91977C19.7058 9.5447 19.1971 9.33398 18.6667 9.33398Z" stroke="#292929" stroke-linecap="round" stroke-linejoin="round"/>
<path d="M7.66714 3.00008L7.66714 6.33341C7.73114 7.09875 7.13914 7.69875 6.33381 7.66675L3.00047 7.66675M1.00047 1.00008L6.33381 6.33341" stroke="#292929" stroke-linecap="round" stroke-linejoin="round"/>
</svg>`;
    innerDiv1.appendChild(innericondiv1);
    var innerDiv2 = document.createElement('div');
    var innericondiv12 = document.createElement('div');
    innericondiv12.classList.add('logo-class');
    innerDiv2Text.classList.add('header-text');
    innerDiv2.classList.add('inner-div-2', 'font-style');
    innerDiv2Text.textContent = 'تـــاویــا';
    innericondiv12.innerHTML = '<img src="https://learn.taavsys.com/chat/icons/logo.png">';
    innerDiv2.appendChild(innericondiv12);
    innerDiv2.appendChild(innerDiv2Text);

    Div1.appendChild(innerDiv1);
    Div1.appendChild(innerDiv2);

    innerDiv1.addEventListener('click', toggleChatBox);
    return Div1;
}


function StartingChat(InputName, InputNumber) {

    // ساخت اتصال کاربر
    RConnection(InputName.value, InputNumber.value);
    if (localStorage.getItem('ChatID')) {
        console.log('javab dad');

    } else {
        RCreateChat(ProductId);
    }
    var section2 = document.querySelector('.chatbox-inside-section2');
    var section3 = document.querySelector('.chatbox-inside-section3');


    if (section2 && section3) {
        section2.style.display = 'none';
        section3.style.display = 'flex';
        chatWrapper.style.display = 'flex';
    }

}


var InputName = document.createElement('input');
let WelcomingMsg = document.createElement('div');

function BotWelcoming() {
    WelcomingMsg.classList.add('welcome-msg');
    if (InputName.value === "") {
        if (lang === 'fa' || lang === 'fa-IR') {
            WelcomingMsg.textContent = `کاربر عزیز شما درحال صحبت با تاویا هستید`;
        } else if (lang.includes('en')) { // Check if 'lang' includes 'en'
            WelcomingMsg.textContent = `Dear user you're talking to Taavia`;
        }
    } else {
        if (lang === 'fa' || lang === 'fa-IR') {
            WelcomingMsg.textContent = `${InputName.value} عزیز شما درحال صحبت با تاویا هستید`;
        } else if (lang.includes('en')) { // Check if 'lang' includes 'en'
            WelcomingMsg.textContent = `Dear ${InputName.value} you're talking to Taavia`;
        }
    }



    chatWrapper.appendChild(WelcomingMsg);
}


// Function to create the second section (form and inputs)
var PinsideSection2 = document.createElement('p');
var InputNumber = document.createElement('input');
var StartChatBtn = document.createElement('button');
function createChatboxForm() {
    var Div2 = document.createElement('div');
    Div2.classList.add('chatbox-inside-section2');

    var InnerInsideSection2Form = document.createElement('form');
    InnerInsideSection2Form.classList.add('InnerInsideSection2Form');

    PinsideSection2.textContent = 'لطفا برای ارتباط بهتر نام و شماره خود را وارد کنید';
    InputName.type = 'text';
    InputName.placeholder = 'نام شما';


    InputNumber.type = 'tel';
    // InputNumber.pattern = '[0-9]{11}';
    InputNumber.placeholder = 'شماره موبایل';

    StartChatBtn.textContent = 'شروع چت';
    StartChatBtn.type = 'submit';
    StartChatBtn.style.backgroundColor = settingsData?.buttonSetting?.color;
    StartChatBtn.style.color = settingsData?.chatSetting?.userMessageTextColor;

    InnerInsideSection2Form.appendChild(InputName);
    InnerInsideSection2Form.appendChild(InputNumber);
    InnerInsideSection2Form.appendChild(StartChatBtn);



    Div2.appendChild(PinsideSection2);
    Div2.appendChild(InnerInsideSection2Form);



    // Form submission event to dynamically add messages and show section 3
    StartChatBtn.addEventListener('click', function (event) {
        event.preventDefault();
        BotWelcoming();
        StartingChat(InputName, InputNumber);
    });


    return Div2;
}


function DisableSendMsg(isDisabled) {
    const chatboxSection = document.querySelector('.chatbox-inside-section3');
    var textarea = Div3.querySelector('textarea');

    if (isDisabled && textarea) {
        chatboxSection.style.pointerEvents = 'none';
        chatboxSection.style.opacity = '0.5';
        textarea.style.opacity = '0.5';
        textarea.disabled = true;

    } else {
        chatboxSection.style.pointerEvents = 'auto';
        chatboxSection.style.opacity = '1';
        textarea.disabled = false;
        textarea.style.opacity = '1';


    }


}



// Function to create the third section (input and icons)
var Div3 = document.createElement('div');
var MessageInput = document.createElement('textarea');
failureMessageSent = true;
function createChatboxMessageContainer() {
    Div3.classList.add('chatbox-inside-section3');
    Div3.style.position = 'relative';
    var InnerInsideSection31 = document.createElement('div');
    var InnerInsideSection32 = document.createElement('div');

    var SendIcon = document.createElement('div');
    SendIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><!--!Font Awesome Pro 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path d="M0 288l37 15.9 155 66.4s0 0 0 0l0 45.7 0 72 64 24 55.7-90.4-29.7-12.7-39.4 64-18.6-7 0-49.8 0-32s0 0 0 0l193.5 82.9L448 480l4.4-32.9L506.7 39.7 512 0 477.1 19.6 35.1 268.2 0 288zm239.4 67.8L468 87.7 421.9 434 239.4 355.8zM430.2 82.7L208.6 342.6 72.2 284.1l358-201.4z"/></svg>`;
    SendIcon.classList.add('send-icon');
    SendIcon.style.fill = settingsData?.buttonSetting?.color;



    var RecordVoiceIcon = document.createElement('div');
    RecordVoiceIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 384 512"><!--!Font Awesome Pro 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path d="M240 128l0 32-32 0c-8.8 0-16 7.2-16 16s7.2 16 16 16l32 0 0 32-32 0c-8.8 0-16 7.2-16 16s7.2 16 16 16l32 0c0 26.5-21.5 48-48 48s-48-21.5-48-48l0-160c0-26.5 21.5-48 48-48s48 21.5 48 48l-32 0c-8.8 0-16 7.2-16 16s7.2 16 16 16l32 0zM96 96l0 160c0 53 43 96 96 96s96-43 96-96l0-160c0-53-43-96-96-96S96 43 96 96zM64 216c0-13.3-10.7-24-24-24s-24 10.7-24 24l0 40c0 89.1 66.2 162.7 152 174.4l0 33.6-48 0c-13.3 0-24 10.7-24 24s10.7 24 24 24l72 0 72 0c13.3 0 24-10.7 24-24s-10.7-24-24-24l-48 0 0-33.6c85.8-11.7 152-85.3 152-174.4l0-40c0-13.3-10.7-24-24-24s-24 10.7-24 24l0 40c0 70.7-57.3 128-128 128s-128-57.3-128-128l0-40z"/></svg>`;
    RecordVoiceIcon.classList.add('recordvoice-icon');
    RecordVoiceIcon.style.fill = settingsData?.buttonSetting?.color;

    var TrashVoiceIcon = document.createElement('div');
    TrashVoiceIcon.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 448 512"><!--!Font Awesome Pro 6.6.0 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license (Commercial License) Copyright 2024 Fonticons, Inc.--><path d="M164.2 39.5L148.9 64l150.3 0L283.8 39.5c-2.9-4.7-8.1-7.5-13.6-7.5l-92.5 0c-5.5 0-10.6 2.8-13.6 7.5zM311 22.6L336.9 64 384 64l32 0 16 0c8.8 0 16 7.2 16 16s-7.2 16-16 16l-16 0 0 336c0 44.2-35.8 80-80 80l-224 0c-44.2 0-80-35.8-80-80L32 96 16 96C7.2 96 0 88.8 0 80s7.2-16 16-16l16 0 32 0 47.1 0L137 22.6C145.8 8.5 161.2 0 177.7 0l92.5 0c16.6 0 31.9 8.5 40.7 22.6zM64 96l0 336c0 26.5 21.5 48 48 48l224 0c26.5 0 48-21.5 48-48l0-336L64 96zm80 80l0 224c0 8.8-7.2 16-16 16s-16-7.2-16-16l0-224c0-8.8 7.2-16 16-16s16 7.2 16 16zm96 0l0 224c0 8.8-7.2 16-16 16s-16-7.2-16-16l0-224c0-8.8 7.2-16 16-16s16 7.2 16 16zm96 0l0 224c0 8.8-7.2 16-16 16s-16-7.2-16-16l0-224c0-8.8 7.2-16 16-16s16 7.2 16 16z"/></svg>`;
    TrashVoiceIcon.classList.add('Trash-icon');
    TrashVoiceIcon.style.fill = settingsData?.buttonSetting?.color;
    TrashVoiceIcon.style.display = 'none'; // Hide Trash icon initially

    // Create the textarea for text input
    MessageInput.placeholder = 'یک سوال دیگر نیز داشتم...';

    // Create the canvas for frequency visualization (initially hidden)
    var canvas = document.createElement('canvas');
    canvas.width = 200;
    canvas.height = 40;
    canvas.classList.add('frequency-visualizer');
    var canvasContext = canvas.getContext('2d');
    canvas.style.display = 'none';

    // Create duration display for recording time
    var durationDisplay = document.createElement('span');
    durationDisplay.classList.add('duration-display');
    durationDisplay.style.marginLeft = '10px';
    durationDisplay.style.display = 'none';

    InnerInsideSection31.appendChild(SendIcon);
    InnerInsideSection31.appendChild(RecordVoiceIcon);
    InnerInsideSection31.appendChild(TrashVoiceIcon);
    InnerInsideSection31.classList.add('InnerInsideSection31');

    InnerInsideSection32.classList.add('InnerInsideSection32');
    InnerInsideSection32.appendChild(MessageInput);
    InnerInsideSection32.appendChild(canvas);
    InnerInsideSection32.appendChild(durationDisplay);


    Div3.appendChild(InnerInsideSection31);
    Div3.appendChild(InnerInsideSection32);


    let mediaRecorder;
    let audioChunks = [];
    let isRecording = false;
    const messageElement = document.createElement('div');



    let audioContext;
    let analyser;
    let dataArray;
    let bufferLength;
    let visualizationInterval;
    let recordingInterval;
    let recordingStartTime;

    function formatDuration(seconds) {
        const minutes = Math.floor(seconds / 60);
        const remainingSeconds = Math.floor(seconds % 60);
        return `${minutes}:${remainingSeconds < 10 ? '0' : ''}${remainingSeconds}`;
    }


    function visualizeFrequency() {
        analyser.getByteTimeDomainData(dataArray);

        const shiftAmount = 2;
        const imageData = canvasContext.getImageData(shiftAmount, 0, canvas.width - shiftAmount, canvas.height);
        canvasContext.clearRect(0, 0, canvas.width, canvas.height);
        canvasContext.putImageData(imageData, 0, 0);

        const gradient = canvasContext.createLinearGradient(0, 0, 0, canvas.height);
        gradient.addColorStop(0, settingsData?.chatSetting?.userMessageColor);
        gradient.addColorStop(1, settingsData?.chatSetting?.userMessageColor);

        canvasContext.lineWidth = 3;
        canvasContext.strokeStyle = gradient;

        const sliceWidth = canvas.width / bufferLength;
        let x = canvas.width - shiftAmount;

        const silenceThreshold = 10;
        let isSilent = true;

        for (let i = 0; i < bufferLength; i++) {
            const normalizedValue = Math.abs(dataArray[i] - 128);
            if (normalizedValue > silenceThreshold) {
                isSilent = false;
                break;
            }
        }

        if (isSilent) {
            canvasContext.beginPath();
            canvasContext.moveTo(x, canvas.height / 2);
            canvasContext.lineTo(x + shiftAmount, canvas.height / 2);
            canvasContext.stroke();
        } else {
            canvasContext.beginPath();
            const lineSpacing = 3;
            for (let i = 0; i < bufferLength; i++) {
                const normalizedValue = (dataArray[i] - 128) / 128.0;
                const maxLineLength = 150;
                const minLineLength = 50;
                const lineLength = (normalizedValue < 0 ? minLineLength : maxLineLength) * Math.abs(normalizedValue);

                const y = (canvas.height / 2) - (lineLength / 2);

                if (i % 2 === 0) {
                    canvasContext.moveTo(x, (canvas.height / 2) - (lineLength / 2));
                    canvasContext.lineTo(x, (canvas.height / 2) + (lineLength / 2));
                } else {
                    const shortLineLength = lineLength / 2;
                    canvasContext.moveTo(x, (canvas.height / 2) - (shortLineLength / 2));
                    canvasContext.lineTo(x, (canvas.height / 2) + (shortLineLength / 2));
                }

                x += sliceWidth + lineSpacing;
            }

            canvasContext.stroke();
        }
    }

    // Start recording and display canvas when clicking RecordVoiceIcon
    RecordVoiceIcon.addEventListener('click', async () => {
        if (!isRecording) {
            try {
                const stream = await navigator.mediaDevices.getUserMedia({ audio: true });

                audioContext = new AudioContext();
                analyser = audioContext.createAnalyser();
                const source = audioContext.createMediaStreamSource(stream);
                source.connect(analyser);

                analyser.fftSize = 2048;
                bufferLength = analyser.frequencyBinCount;
                dataArray = new Uint8Array(bufferLength);

                // Show canvas and duration display
                canvas.style.display = 'block';
                durationDisplay.style.display = 'inline-block';

                if (!InnerInsideSection32.contains(canvas)) {
                    InnerInsideSection32.appendChild(canvas);
                }
                if (!InnerInsideSection32.contains(durationDisplay)) {
                    InnerInsideSection32.appendChild(durationDisplay);
                }

                // Visualize frequency
                visualizationInterval = setInterval(visualizeFrequency, 100);

                mediaRecorder = new MediaRecorder(stream);

                // Start duration timer
                recordingStartTime = Date.now();
                recordingInterval = setInterval(() => {
                    const elapsedSeconds = Math.floor((Date.now() - recordingStartTime) / 1000);
                    durationDisplay.textContent = formatDuration(elapsedSeconds);
                }, 500);

                mediaRecorder.ondataavailable = event => {
                    audioChunks.push(event.data);
                };

                mediaRecorder.start();
                isRecording = true;

                // Switch icons
                RecordVoiceIcon.style.display = 'none';
                TrashVoiceIcon.style.display = 'block';
                MessageInput.style.display = 'none';

            } catch (error) {
                messageElement.textContent = "Microphone access denied. Please allow access.";
            }
        }
    });

    SendIcon.addEventListener('click', () => {
        if (isRecording) {
            mediaRecorder.stop();
            isRecording = false;

            mediaRecorder.onstop = async () => { // Mark as async
                clearInterval(visualizationInterval);
                clearInterval(recordingInterval);

                const audioBlob = new Blob(audioChunks, { type: 'audio/mp3' });
                const audioUrl = URL.createObjectURL(audioBlob);
                const duration = Math.floor((Date.now() - recordingStartTime) / 1000);
                console.log(audioUrl);
                console.log(audioBlob.type);
                const currentTime = new Date();
                // sendMessage(audioBlob, 'user-message', true, duration);
                console.log(audioBlob);
                // Reset UI
                audioChunks = [];
                canvasContext.clearRect(0, 0, canvas.width, canvas.height);
                MessageInput.style.display = 'block';
                canvas.style.display = 'none';
                durationDisplay.style.display = 'none';
                RecordVoiceIcon.style.display = 'block';
                TrashVoiceIcon.style.display = 'none';

                try {
                    let docId = await uploadAudioFile(audioBlob); // Wait for documentId
                    console.log('docId hasttttttttttttttttttttttt: ', docId);
                    if (docId) {
                        RSendVoice(docId);
                        UserChat(audioBlob, currentTime, null, docId);
                        ShowTypingIndicator('on');
                        DisableSendMsg(true);
                        localStorage.setItem('failureMessageSent', 'true');


                    } else {
                        console.error('Failed to retrieve documentId from uploadAudioFile.');
                    }
                } catch (err) {
                    console.error('Error uploading audio file:', err);
                }
            };

        } else if (MessageInput.value.trim() !== "") {
            const currentTime = new Date();
            UserChat(MessageInput.value, currentTime);
            RSendMessage(MessageInput.value);
            localStorage.setItem('failureMessageSent', 'true');
            ShowTypingIndicator('on');
            DisableSendMsg(true);
            MessageInput.value = '';
            RecordVoiceIcon.style.display = 'block';
            TrashVoiceIcon.style.display = 'none';

        }

    });



    MessageInput.addEventListener('keydown', function (event) {
        if (event.key === 'Enter') {
            // Prevent default behavior
            event.preventDefault();

            // If shift+enter is pressed, go to a new line, but only if the current line isn't empty
            if (event.shiftKey) {
                // Check if current value without trimming spaces or newlines
                const lines = MessageInput.value.split('\n');
                const currentLine = lines[lines.length - 1]; // Get the current line before pressing Enter

                // Only add newline if current line is not empty or does not consist only of spaces
                if (currentLine.trim() !== '') {
                    MessageInput.value += "\n";
                }
            } else {
                // Send the message if not recording and message is not empty
                if (MessageInput.value.trim() !== "") {
                    SendIcon.click(); // Trigger the click event for SendIcon to send the message
                }
            }
        }
        startTimestampUpdater();
    });




    // Stop recording and delete data when clicking TrashVoiceIcon
    TrashVoiceIcon.addEventListener('click', () => {
        if (isRecording) {
            mediaRecorder.stop();
            isRecording = false;

            clearInterval(visualizationInterval);
            clearInterval(recordingInterval);

            // Clear audio data and reset the UI
            audioChunks = [];
            canvasContext.clearRect(0, 0, canvas.width, canvas.height);

            // Reset to default UI state
            MessageInput.style.display = 'block';
            canvas.style.display = 'none';
            durationDisplay.style.display = 'none';

            // Switch icons back to default
            RecordVoiceIcon.style.display = 'block';
            TrashVoiceIcon.style.display = 'none';
        }
    });


    return Div3;
}




async function uploadAudioFile(audioBlob) {
    // آدرس API و تنظیمات لازم
    const apiEndpoint = BaseUri + '/api/v1/storages';
    const tenantId = TenantId; // مقدار tenantId مورد نیاز

    // ایجاد فرم‌دیتا و اضافه کردن فایل ضبط‌شده
    const formData = new FormData();
    formData.append('file', audioBlob, 'recording.mp3'); // نام فایل ضبط‌شده

    try {
        // ارسال درخواست POST به API
        const response = await fetch(apiEndpoint, {
            method: 'POST',
            headers: {
                'tenantId': tenantId,
            },
            body: formData // ارسال فرم‌دیتا به عنوان بدنه
        });


        if (!response.ok) {
            throw new Error(`HTTP error! Status: ${response.status}`);
        }


        const result = await response.text(); // یا response.json() بسته به نوع پاسخ
        console.log('API Response:', result); // نمایش پاسخ در کنسول
        return result;
    } catch (error) {
        // مدیریت خطاها
        console.error('Error uploading audio file:', error);
    }
}



// function sendMessage(audioBlob, messageClass, isVoice, duration = 0) {
//     var messageContainer = document.createElement('div');
//     messageContainer.classList.add('message-container');

//     if (isVoice) {

//         var audioContainer = document.createElement('div');
//         audioContainer.classList.add('audio-container');
//         // audioContainer.style.backgroundColor = settingsData.chatSetting.userMessageColor;

//         // var playButton = document.createElement('button');
//         // playButton.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path d="M480 256A224 224 0 1 0 32 256a224 224 0 1 0 448 0zM0 256a256 256 0 1 1 512 0A256 256 0 1 1 0 256zM208 364.3L176 384l0-37.6 0-180.9 0-37.6 32 19.7 145.5 89.5L384 256l-30.5 18.8L208 364.3zM322.9 256L208 185.3l0 141.5L322.9 256z "/></svg>`;
//         // playButton.classList.add('play-button');
//         // playButton.style.width = '30px';
//         // playButton.style.height = '30px';
//         // playButton.style.cursor = 'pointer';
//         // playButton.style.background = 'none';
//         // playButton.style.border = 'none';
//         // playButton.setAttribute('aria-label', 'Play audio');

//         // var pauseButton = document.createElement('button');
//         // pauseButton.innerHTML = `<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512"><path d="M480 256A224 224 0 1 0 32 256a224 224 0 1 0 448 0zM0 256a256 256 0 1 1 512 0A256 256 0 1 1 0 256zm224-80l0 160c0 8.8-7.2 16-16 16s-16-7.2-16-16l0-160c0-8.8 7.2-16 16-16s16 7.2 16 16zm96 0l0 160c0 8.8-7.2 16-16 16s-16-7.2-16-16l0-160c0-8.8 7.2-16 16-16s16 7.2 16 16z "/></svg>`;
//         // pauseButton.classList.add('pause-button');
//         // pauseButton.style.width = '30px';
//         // pauseButton.style.height = '30px';
//         // pauseButton.style.cursor = 'pointer';
//         // pauseButton.style.background = 'none';
//         // pauseButton.style.border = 'none';
//         // pauseButton.style.display = 'none';
//         // pauseButton.setAttribute('aria-label', 'Pause audio');

//         // const svgPath = playButton.querySelector('svg path');
//         // const svgPath2 = pauseButton.querySelector('svg path');

//         // if (svgPath) {
//         //     svgPath.setAttribute('fill', settingsData?.chatSetting?.userMessageTextColor);
//         // }

//         // if (svgPath2) {
//         //     svgPath2.setAttribute('fill', settingsData?.chatSetting?.userMessageTextColor);
//         // }

//         var audio = document.createElement('div');
//         let audioUrl = URL.createObjectURL(audioBlob);
//         audio.innerHTML = `<audio controls> <source src="${audioUrl}" type="audio/mp3"></audio>`;
//         // audio.src = audioUrl;


//         // var canvas = document.createElement('canvas');
//         // canvas.classList.add('waveform-canvas');

//         // var durationText = document.createElement('span');
//         // durationText.classList.add('voice-duration');
//         // durationText.textContent = formatDuration(duration);
//         // durationText.style.color = settingsData.chatSetting.userMessageTextColor;

//         // audioContainer.appendChild(playButton);
//         // audioContainer.appendChild(pauseButton);
//         // audioContainer.appendChild(canvas);
//         // audioContainer.appendChild(durationText);

//         messageContainer.appendChild(audioContainer);
//         audioContainer.appendChild(audio);
//         chatWrapper.appendChild(messageContainer);
//         chatWrapper.scrollTop = chatWrapper.scrollHeight;

//         // drawWaveform(audioUrl, canvas);

//         // playButton.addEventListener('click', function () {
//         //     audio.play();
//         //     playButton.style.display = 'none';
//         //     pauseButton.style.display = 'block';
//         // });

//         // pauseButton.addEventListener('click', function () {
//         //     audio.pause();
//         //     pauseButton.style.display = 'none';
//         //     playButton.style.display = 'block';
//         // });

//         // audio.addEventListener('ended', function () {
//         //     pauseButton.style.display = 'none';
//         //     playButton.style.display = 'block';
//         // });



//         // // Add click event listener to canvas
//         // canvas.addEventListener('click', function (event) {
//         //     var clickX = event.offsetX; // Position of the click in relation to the canvas
//         //     var canvasWidth = canvas.width; // Width of the waveform canvas

//         //     // Ensure the audio duration is properly mapped to the canvas width
//         //     var clickTime = (clickX / canvasWidth) * audio.duration;

//         //     // If click is near the end of the canvas (to avoid inaccuracies near the end)
//         //     if (clickTime >= audio.duration) {
//         //         audio.currentTime = audio.duration; // Jump to the very end
//         //     } else {
//         //         audio.currentTime = clickTime; // Jump to the appropriate time based on the click position
//         //     }
//         // });

//         // Update voice duration during playback
//         // audio.addEventListener('timeupdate', function () {
//         //     durationText.textContent = formatDuration(audio.currentTime);
//         // });
//     }
// }


// function drawWaveform(audioUrl, canvas, audio) {
//     var canvasCtx = canvas.getContext('2d');
//     var audioContext = new (window.AudioContext || window.webkitAudioContext)();
//     var analyser = audioContext.createAnalyser();
//     analyser.fftSize = 2048;
//     var bufferLength = analyser.fftSize;
//     var dataArray = new Uint8Array(bufferLength);

//     fetch(audioUrl)
//         .then(response => response.arrayBuffer())
//         .then(arrayBuffer => audioContext.decodeAudioData(arrayBuffer))
//         .then(audioBuffer => {
//             var rawData = audioBuffer.getChannelData(0);
//             var samples = 2500; // This determines how many samples to plot
//             var blockSize = Math.floor(rawData.length / samples);
//             var filteredData = [];
//             for (var i = 0; i < samples; i++) {
//                 var blockStart = blockSize * i;
//                 var sum = 0;
//                 for (var j = 0; j < blockSize; j++) {
//                     sum += Math.abs(rawData[blockStart + j]);
//                 }
//                 filteredData.push(sum / blockSize);
//             }

//             var max = Math.max(...filteredData);
//             filteredData = filteredData.map(n => n / max);

//             // Ensure the canvas is cleared and properly sized
//             canvas.width = canvas.parentElement.offsetWidth;  // Make canvas width match container width

//             canvasCtx.clearRect(0, 0, canvas.width, canvas.height);
//             canvasCtx.strokeStyle = settingsData.chatSetting.userMessageTextColor;
//             canvasCtx.lineWidth = 2;

//             canvasCtx.beginPath();
//             var sliceWidth = canvas.width / samples;
//             var x = 0;

//             for (var i = 0; i < samples; i++) {
//                 var y = (1 - filteredData[i]) * canvas.height / 2;
//                 if (i === 0) {
//                     canvasCtx.moveTo(x, y);
//                 } else {
//                     canvasCtx.lineTo(x, y);
//                 }
//                 x += sliceWidth;
//             }

//             canvasCtx.stroke();
//         })
//         .catch(e => {
//             console.error(e);
//             var errorText = document.createElement('span');
//             errorText.textContent = 'Unable to load audio.';
//             canvas.parentElement.appendChild(errorText);
//         });
// }


// Utility function to format duration



function formatDuration(seconds) {
    const minutes = Math.floor(seconds / 60);
    const remainingSeconds = Math.floor(seconds % 60);
    return `${minutes}:${remainingSeconds < 10 ? '0' : ''}${remainingSeconds}`;
}



window.addEventListener('resize', function () {
    const chatIcon = document.querySelector('.chat-icon');
    const chatbox = document.querySelector('.chatbox');

    if (window.innerWidth <= 530 && chatbox && chatbox.style.display === 'flex') {
        chatIcon.style.display = 'none';
    } else {
        chatIcon.style.display = 'flex';
    }
});


async function chatbox_EN() {
    if (lang.includes('en')) {
        const chatboxes = document.getElementsByClassName('chatbox');
        for (let chatbox of chatboxes) {
            chatbox.style.direction = 'rtl';
            chatbox.style.fontFamily = 'Rubik';
        }

        const innerdiv = document.getElementsByClassName('inner-div-1');
        for (let innerdives of innerdiv) {
            innerdives.style.marginRight = '15px';
        }


        const timestampstatus = document.querySelectorAll('.timestamp-statusicon');
        for (let timestampstatuses of timestampstatus) {
            timestampstatuses.style.justifyContent = 'right';
            timestampstatuses.style.gap = '5px';

        }


        const input = document.querySelectorAll('.InnerInsideSection2Form input');
        for (let inputs of input) {
            inputs.style.direction = 'ltr';
        }


        const logo = document.getElementsByClassName('sign-logo');
        for (let logos of logo) {
            logos.style.width = '50px';
        }


        const timestamp = document.querySelectorAll('.message-timestamp, .bot-message-timestamp');
        for (let timestamps of timestamp) {
            timestamps.style.direction = 'ltr';
            timestamps.style.marginLeft = '5px';
        }

        const bottimestamp = document.querySelectorAll('.bot-reaction-timestamp-container');
        for (let bottimestamps of bottimestamp) {
            bottimestamps.style.marginRight = '5px';
        }

        const typingindicator = document.querySelectorAll('.typing-indicator');
        for (let typingindicators of typingindicator) {
            typingindicators.style.direction = 'ltr';
        }

        const botmessage = document.querySelectorAll('.bot-message');
        for (let botmessages1 of botmessage) {
            botmessages1.style.setProperty('direction', 'ltr', 'important');
        }





        SignLogo.src = 'https://learn.taavsys.com/chat/icons/Artboard%209.png';
        InputName.placeholder = 'Name';
        InputNumber.placeholder = 'Number';
        StartChatBtn.textContent = 'Start';
        innerDiv2Text.textContent = 'Taavia';
        companySign.innerHTML = ' Powered by ';
        PinsideSection2.textContent = 'Please fill the form for better communication';
        MessageInput.placeholder = 'I have another question...';
        MessageInput.style.direction = 'ltr';
        chatWrapper.style.direction = 'ltr';
        TypingIndicator.textContent = 'Taavia is typing...';



    }
}


async function initializeChatbox() {
    console.log("تابع initializeChatbox فراخوانی شد");
    addStyles();
    var iconDiv = await createChatIcon();
    document.body.appendChild(iconDiv);
    var chatBox = await createChatBox();
    console.log('chatbox created!');
    iconDiv.appendChild(chatBox);
    await chatbox_EN();
}

initializeChatbox();