<?php
/*
Plugin Name: Taavia
Description: A plugin for adding a support system to the website.
Version: 1.1.10
Author: Taavsys
Author URI: https://taavsys.com
Text Domain: taavia
Domain Path: /languages
Requires at least: 6.0.0
Requires PHP: 7.4
*/


if (!defined('ABSPATH')) {
    exit;
}


function taavia_check_requirements() {
    global $wp_version;

    $required_wp_version = '6.0.0';
    $required_php_version = '7.4';

    if (version_compare(PHP_VERSION, $required_php_version, '<')) {
        deactivate_plugins(plugin_basename(__FILE__)); // افزونه را غیرفعال می‌کند
        wp_die(
            sprintf(
                __('Taavia requires at least PHP %s. Your current PHP version is %s.', 'taavia'),
                $required_php_version,
                PHP_VERSION
            ),
            __('Plugin Activation Error', 'taavia'),
            array('back_link' => true)
        );
    }

    if (version_compare($wp_version, $required_wp_version, '<')) {
        deactivate_plugins(plugin_basename(__FILE__)); // افزونه را غیرفعال می‌کند
        wp_die(
            sprintf(
                __('Taavia requires at least WordPress %s. Your current WordPress version is %s.', 'taavia'),
                $required_wp_version,
                $wp_version
            ),
            __('Plugin Activation Error', 'taavia'),
            array('back_link' => true)
        );
    }
}
register_activation_hook(__FILE__, 'taavia_check_requirements');



function zhenobot_add_admin_menu() {
    add_menu_page(
        __('Taavia Settings', 'taavia'), 
        __('Taavia', 'taavia'), 
        'manage_options', 
        'taavia-settings', 
        'taavia_render_settings_page',
        plugin_dir_url(__FILE__) . 'assets/img/logo.svg', 
        120
    );

    add_submenu_page(
        'taavia-settings', 
        __('Customization', 'taavia'), 
        __('Customize Plugin', 'taavia'), 
        'manage_options', 
        'taavia-customization', 
        'taavia_render_customization_page'
    );
}
function taavia_load_textdomain() {
    load_plugin_textdomain('taavia', false, dirname(plugin_basename(__FILE__)) . '/languages/');
}
add_action('plugins_loaded', 'taavia_load_textdomain');

function taavia_translate_plugin_info() {
    __('Taavia', 'taavia');
    __('A plugin for adding a support system to the website.', 'taavia');
    __('Taavsys', 'taavia');
}
add_action('plugins_loaded', 'taavia_translate_plugin_info');

add_action('admin_menu', 'zhenobot_add_admin_menu');





function taavia_render_settings_page() {
    $content = plugin_dir_path(__FILE__) . 'templates/settings-page.php';
    include plugin_dir_path(__FILE__) . 'templates/base-template.php';
}

function taavia_render_customization_page() {
    $content = plugin_dir_path(__FILE__) . 'templates/customization-page.php';
    include plugin_dir_path(__FILE__) . 'templates/base-template.php';
}

// Function to add scripts to WordPress
function enqueue_custom_js_files_for_zenobot() {
    wp_enqueue_script('signal-js', plugin_dir_url(__FILE__) . 'assets/js/SignalR.js', array(), '1.0', true);
    wp_enqueue_script('chat-js', plugin_dir_url(__FILE__) . 'sample/plugin.js', array(), '1.0', true);
}

// Hook to add scripts to the frontend of the site
add_action('wp_enqueue_scripts', 'enqueue_custom_js_files_for_zenobot');

function taavia_admin_enqueue_styles() {
    wp_enqueue_style( 'taavia-admin-style', plugin_dir_url( __FILE__ ) . 'assets/css/admin-style.css' );
}
add_action( 'admin_enqueue_scripts', 'taavia_admin_enqueue_styles' );
